import Swal from 'sweetalert2';
import { fetchConToken } from '../helpers/fetch';
import { types } from '../types/types';

export const startDeletingBus = (data) => {
    const {_id} = data;
    return async (dispatch) =>{
        fetchConToken(`buses/${ _id }`, data, 'DELETE')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "¡Perfecto! Se borra.", 'success');
                console.log("Envío el _id: ", _id);
                return dispatch(deleteBus(_id))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }               
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }
}

export const deleteBus = (busId) => {
    return {
        type: types.busDeleted,
        payload: busId
    }
}

export const startEditingBus = (data) => {
    return async (dispatch) =>{
        fetchConToken(`buses/${ data._id }`, data, 'PUT')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "¡Perfecto! Se actualiza.", 'success');
                return dispatch(busEdited(data.data))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }               
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }
}


const busEdited = (bus) => {
    return {
        type: types.busEdited,
        payload: bus
    }
}

export const busSavingNew = (data) => {
    return async (dispatch) =>{
        fetchConToken("buses", data, 'POST')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', 'Perfecto! Se Guarda', 'success');
                return dispatch(busSaved(data.data))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }                
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
        
    }    
}

const busSaved = (bus) => ({
    type: types.busSaved,
    payload: bus
})

export const busesStartLoading = () => {
    return async (dispatch) =>{
        fetchConToken("buses")
        .then(response => response.json())
        .then(data => {
                if(data.ok){
                    return dispatch(busesLoaded(data.data))
                }
                else {
                    let errDesc = '<ul>';
                    data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                    data?.error && (errDesc += '<li>'+data?.error+'</li>');
                    if(data?.errors){
                        for(let campo in data.errors){
                            errDesc += '<li>'+data.errors[campo].msg + "</li>"
                        }
                    }
                    errDesc += '</ul>';
                    Swal.fire({
                        icon: 'error',
                        title: 'Oops...',
                        html: errDesc, 
                        footer: 'Habla con el administrador'                   
                    })
                }
            }
        )
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }    
}

const busesLoaded = (buses) => ({
    type: types.busesLoaded,
    payload: buses
})
