import { fetchConToken } from '../helpers/fetch';
import { types } from '../types/types';
import Swal from 'sweetalert2';

export const addFavorite = (data) => {    
    return async (dispatch) =>{
        
        fetchConToken("favorites",  data, 'POST')
        .then(response => response.json())
        .then(data => {
                localStorage.setItem('favoriteLines', JSON.stringify(data.data ));
                if(data.ok){
                    return dispatch(favoriteAdded(data.data))    
                }
                else {
                    let errDesc = '<ul>';
                    data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                    data?.error && (errDesc += '<li>'+data?.error+'</li>');
                    if(data?.errors){
                        for(let campo in data.errors){
                            errDesc += '<li>'+data.errors[campo].msg + "</li>"
                        }
                    }
                    errDesc += '</ul>';
                    Swal.fire({
                        icon: 'error',
                        title: 'Oops...',
                        html: errDesc, 
                        footer: 'Habla con el administrador'                   
                    })
                }
            }
        )
        .catch((error) => {
            console.log(error)
        });
    }    
}

export const favoriteAdded = (data) => ({
    type: types.favoriteLineAdded,
    payload: data    
})

export const delFavorite = (data) => {    
    return async (dispatch) =>{
        console.log("Envío la línea ", data)
        fetchConToken(`favorites/${data}`,  {}, 'DELETE')
        .then(response => response.json())
        .then(data => {
                localStorage.setItem('favoriteLines', JSON.stringify(data.data ));
                    if(data.ok){
                        return dispatch(favoriteRemoved(data.data))    
                    }
                    else {
                        let errDesc = '<ul>';
                        data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                        data?.error && (errDesc += '<li>'+data?.error+'</li>');
                        if(data?.errors){
                            for(let campo in data.errors){
                                errDesc += '<li>'+data.errors[campo].msg + "</li>"
                            }
                        }
                        errDesc += '</ul>';
                        Swal.fire({
                            icon: 'error',
                            title: 'Oops...',
                            html: errDesc, 
                            footer: 'Habla con el administrador'                   
                        })
                    }
            }
        )
        .catch((error) => {
            console.log(error)
        });
    }    
}

export const favoriteRemoved = (data) => ({
    type: types.favoriteLineRemoved,
    payload: data    
})

export const getFavorites = (data) => {    
    return async (dispatch) =>{
        
        fetchConToken("favorites")

        .then(response => response.json())
        .then(data => {
                localStorage.setItem('favoriteLines', JSON.stringify(data.data ));
                return dispatch(favoritesLoaded(data.data))    
                
            }
        )
        .catch((error) => {
            console.log(error)
        });
    }    
}

export const favoritesLoaded = (data) => ({
    type: types.favoritesLoaded,
    payload: data    
})
