import Swal from 'sweetalert2';
import { fetchConToken } from '../helpers/fetch';
import { types } from '../types/types';

export const startDeletingLine = (data) => {
    const {_id} = data;
    return async (dispatch) =>{
        fetchConToken(`lines/${ _id }`, data, 'DELETE')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "¡Perfecto! Se borra", 'success');
                console.log("Envío el _id: ", _id);
                return dispatch(deleteLine(_id))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }                 
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }
}

export const deleteLine = (lineId) => {
    return {
        type: types.lineDeleted,
        payload: lineId
    }
}

export const startEditingNew = (data) => {
    return async (dispatch) =>{
        fetchConToken(`lines/${ data._id }`, data, 'PUT')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', 'Perfecto! Se actualiza', 'success');
                return dispatch(editLine(data.data))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }                
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }
}

export const editLine = (line) => {
    return {
        type: types.lineEdited,
        payload: line
    }
}

export const lineSavingNew = (data) => {
    return async (dispatch) =>{
        fetchConToken("lines", data, 'POST')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', '¡Perfecto! Se Guarda.', 'success');
                return dispatch(lineSaved(data.data))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }               
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }    
}

const lineSaved = (line) => ({
    type: types.lineSaved,
    payload: line
})

export const linesNotLoaded = () => ({
    type: types.linesNotLoaded,    
})

export const allLinesLoading = () => {    
    return async (dispatch) =>{
        
        fetchConToken("lines")
        .then(response => response.json()
        )
        .then(data => {
            if(data.ok){
                return dispatch(linesLoaded(data.data))

            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            } 
        })
        .catch((error) => {
            console.log(error)
        });
    }    
}

export const lineNotLoaded = () => ({
    type: types.lineNotLoaded,    
})

const linesLoaded = (lines) => ({
    type: types.linesLoaded,
    payload: lines
})



export const oneLineLoading = (codeLine) => {
    return async (dispatch) =>{
        fetchConToken(`lines/${codeLine}`)
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                return dispatch(lineLoaded(data.data))
            }
            else {
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    text: data.msg, 
                    footer: 'Habla con el administrador'                   
                })
            }
            
             
            }
        )
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }
}

export const lineLoaded = (line) => ({
    type: types.lineLoaded,
    payload: line
})