import Swal from "sweetalert2";
import { fetchConToken } from "../helpers/fetch";
import { types } from "../types/types";

export const startDeletingNotification = (data) => {
    const {_id} = data;
    return async (dispatch) =>{
        fetchConToken(`notifications/${ _id }`, data, 'DELETE')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "Perfecto! Se borra", 'success');
                console.log("Envío el _id: ", _id);
                return dispatch(deleteNotification(_id))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }               
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }
}

export const deleteNotification = (notificationId) => {
    return {
        type: types.notificationDeleted,
        payload: notificationId
    }
}

export const notificationSavingNew = (data) => {
    return async (dispatch) =>{        
        fetchConToken("notifications", data, 'POST')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "Perfecto! Se Guarda:" + JSON.stringify(data.data), 'success');
                return dispatch(notificationSaved(data.data))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }                 
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
        
    }    
}

const notificationSaved = (notification) => ({
    type: types.notificationSaved,
    payload: notification
})

export const notificationsStartLoading = () => {
    return async (dispatch) =>{
        fetchConToken(`notifications`)
        .then(response => response.json())
        .then(data => {
                if (data.ok){
                    return dispatch(notificationsLoaded(data.data))
                }
            }
        )
        .catch((error) => {
            console.log(error)
        });
    }
}


export const notificationsLoading = (data) => {
    return async (dispatch) =>{
        fetchConToken(`notifications/${data}` )
        .then(response => response.json())
        .then(data => {
                if (data.ok){
                    return dispatch(notificationsLoaded(data.data))
                }
            }
        )
        .catch((error) => {
            console.log(error)
        });
    }
}

export const notificationsLoaded = (notification) => ({
    type: types.notificationsLoaded,
    payload: notification
})

export const notificationsReset = () => ({
    type: types.notificationsReset
    
})