import Swal from 'sweetalert2';
import { fetchConToken } from '../helpers/fetch';
import { types } from '../types/types';

export const startDeletingRoute = (data) => {
    const {_id} = data;
    return async (dispatch) =>{
        fetchConToken(`routes/${ _id }`, data, 'DELETE')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "Perfecto! Se borra", 'success');
                console.log("Envío el _id: ", _id);
                return dispatch(deleteRoute(_id))    
            }
            else {
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    text: data.msg, 
                    footer: 'Habla con el administrador'                   
                })
            }                
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }
}

export const deleteRoute = (routeId) => {
    return {
        type: types.routeDeleted,
        payload: routeId
    }
}

export const startEditingRoute = (data) => {
    return async (dispatch) =>{
        fetchConToken(`routes/${ data._id }`, data, 'PUT')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "Perfecto! Se actualiza:", 'success');
                return dispatch(routeEdited(data.data))    
            }
            else {
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    text: data.msg, 
                    footer: 'Habla con el administrador'                   
                })
            }                
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }
}


const routeEdited = (route) => {
    return {
        type: types.routeEdited,
        payload: route
    }
}

export const routeSavingNew = (data) => {
    return async (dispatch) =>{
        //dispatch(routeSaved(data))
        
        fetchConToken("routes", data, 'POST')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "Perfecto! Se Guarda:" + JSON.stringify(data.data), 'success');
                return dispatch(routeSaved(data.data))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }                
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
        
    }    
}

const routeSaved = (route) => ({
    type: types.routeSaved,
    payload: route
})

export const routesStartLoading = () => {
    return async (dispatch) =>{

        fetchConToken("routes")
        .then(response => response.json())
        .then(data => {
                if(data.ok){
                    return dispatch(routesLoaded(data.data))
                }
                else {
                    let errDesc = '<ul>';
                    data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                    data?.error && (errDesc += '<li>'+data?.error+'</li>');
                    if(data?.errors){
                        for(let campo in data.errors){
                            errDesc += '<li>'+data.errors[campo].msg + "</li>"
                        }
                    }
                    errDesc += '</ul>';
                    Swal.fire({
                        icon: 'error',
                        title: 'Oops...',
                        html: errDesc, 
                        footer: 'Habla con el administrador'                   
                    })
                }                             
            }
        )
        .catch((error) => {
            console.log(error)
        });
    }    
}

const routesLoaded = (routes) => ({
    type: types.routesLoaded,
    payload: routes
})
