import Swal from 'sweetalert2';
import { fetchConToken } from '../helpers/fetch';
import { types } from '../types/types';

export const startDeletingStop = (data) => {
    const {_id} = data;
    return async (dispatch) =>{
        fetchConToken(`stops/${ _id }`, data, 'DELETE')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "Perfecto! Se borra", 'success');
                console.log("Envío el _id: ", _id);
                return dispatch(deleteStop(_id))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }                              
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }
}

export const deleteStop = (stopId) => {
    return {
        type: types.stopDeleted,
        payload: stopId
    }
}

export const startEditingNewStop = (data) => {
    return async (dispatch) =>{
        fetchConToken(`stops/${ data._id }`, data, 'PUT')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "Perfecto! Se actualiza:", 'success');
                return dispatch(stopEdited(data.data))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }                              
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }
}


const stopEdited = (stop) => {
    return {
        type: types.stopEdited,
        payload: stop
    }
}

export const stopSavingNew = (data) => {
    return async (dispatch) =>{
       
        fetchConToken("stops", data, 'POST')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "Perfecto! Se Guarda:" + JSON.stringify(data.data), 'success');
                return dispatch(stopSaved(data.data))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }                              
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
        
    }    
}

const stopSaved = (stop) => ({
    type: types.stopSaved,
    payload: stop
})

export const stopsStartLoading = () => {
    return async (dispatch) =>{

        fetchConToken("stops")
        .then(response => response.json())
        .then(data => {
                return dispatch(stopsLoaded(data))
            }
        )
        .catch((error) => {
            console.log(error)
        });
    }    
}

const stopsLoaded = (stops) => ({
    type: types.stopsLoaded,
    payload: stops
})
