import Swal from 'sweetalert2';
import { fetchConToken } from '../helpers/fetch';
import { types } from '../types/types';

export const startDeletingUser = (data) => {
    const {_id} = data;
    return async (dispatch) =>{
        fetchConToken(`users/${ _id }`, data, 'DELETE')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "Perfecto! Se borra", 'success');
                console.log("Envío el _id: ", _id);
                return dispatch(deleteUser(_id))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }                              
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }
}

export const deleteUser = (userId) => {
    return {
        type: types.userDeleted,
        payload: userId
    }
}

export const startEditingUser = (data) => {
    return async (dispatch) =>{
        fetchConToken(`users/${ data._id }`, data, 'PUT')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "Perfecto! Se actualiza:", 'success');
                return dispatch(userEdited(data.data))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }                             
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
    }
}


const userEdited = (user) => {
    return {
        type: types.userEdited,
        payload: user
    }
}

export const userSavingNew = (data) => {
    return async (dispatch) =>{
        fetchConToken("users", data, 'POST')
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                Swal.fire('Info', "Perfecto! Se Guarda:" + JSON.stringify(data.data), 'success');
                return dispatch(userSaved(data.data))    
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }                              
        })
        .catch((error) => {
            Swal.fire('Error', JSON.stringify(error), 'error');
        });
        
    }    
}

const userSaved = (user) => ({
    type: types.userSaved,
    payload: user
})

export const usersStartLoading = () => {
    return async (dispatch) =>{

        fetchConToken("users")
        .then(response => response.json())
        .then(data => {
            if(data.ok){
                return dispatch(usersLoaded(data.data))
            }
            else {
                let errDesc = '<ul>';
                data?.msg && (errDesc += '<li>'+data?.msg+'</li>');
                data?.error && (errDesc += '<li>'+data?.error+'</li>');
                if(data?.errors){
                    for(let campo in data.errors){
                        errDesc += '<li>'+data.errors[campo].msg + "</li>"
                    }
                }
                errDesc += '</ul>';
                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    html: errDesc, 
                    footer: 'Habla con el administrador'                   
                })
            }
        })
        .catch((error) => {
            console.log(error)
        });
    }    
}

const usersLoaded = (users) => ({
    type: types.usersLoaded,
    payload: users
})
