import React, { useEffect } from 'react'
import { useDispatch, useSelector } from 'react-redux';
import { allLinesLoading, startDeletingLine, /*linesStartLoading*/ } from '../../actions/lines';
import M from "materialize-css/dist/js/materialize.min.js";
import "materialize-css/dist/css/materialize.min.css";
import { Link } from 'react-router-dom';
import { CrudLinesForm } from '../ui/admin/CrudLinesForm';
import { stopsStartLoading } from '../../actions/stops';
import { openModal } from '../../actions/modal';
import { startEditFormMode, startNewFormMode } from '../../actions/form';
import { BreadCrumbs } from '../ui/BreadCrumbs';
import Swal from 'sweetalert2';

export const CRUDLinesScreen = () => {
        
    const dispatch = useDispatch();
    const {isDownloaded: linesIsDownloaded, data: lines} = useSelector(state => state.lines); 
    const {isDownloaded: stopsIsDownloaded, data: stops} = useSelector(state => state.stops); 

    useEffect(() => {        
        !linesIsDownloaded && dispatch(allLinesLoading())        
    }, [dispatch, linesIsDownloaded])
    useEffect(() => {        
        !stopsIsDownloaded && dispatch(stopsStartLoading())        
    }, [dispatch, stopsIsDownloaded])

    const edit = (e, line) => {
        e.preventDefault();
        dispatch(openModal(line))
        dispatch(startEditFormMode())
    }

    const deleteLine = (e, line) => {
        e.preventDefault();
        const swalWithBootstrapButtons = Swal.mixin({
            customClass: {
              confirmButton: 'btn btn-success ',
              cancelButton: 'btn btn-danger red mr10'
            },
            buttonsStyling: true
          })
          
          swalWithBootstrapButtons.fire({
            title: '¿Estás seguro?',
            text: "Este proceso es irreversible",
            icon: 'warning',
            showCancelButton: true,
            confirmButtonText: 'Sí, bórralo',
            cancelButtonText: '¡No, cancela!',
            reverseButtons: true
          }).then((result) => {
            if (result.isConfirmed) {
                dispatch(startDeletingLine(line));
              swalWithBootstrapButtons.fire(
                'Borrado',
                'El registro ha sido borrado.',
                'success'
              )
            } else if (
              result.dismiss === Swal.DismissReason.cancel
            ) {
              swalWithBootstrapButtons.fire(
                'Cancelado',
                'El proceso de borrado se ha cancelado',
                'error'
              )
            }
          })
    }

    
    useEffect(() => {
        let elems = document.querySelectorAll('.modal');
        M.Modal.init(elems, {
            inDuration: 250,
            outDuration: 250,
            opacity: 0.5,
            dismissible: false,
            startingTop: "4%",
            endingTop: "10%"
          });
          const floatButton = document.querySelectorAll('.fixed-action-btn');
          M.FloatingActionButton.init(floatButton, {
            direction: 'left',
            hoverEnabled: false
          });
      });

    const handleOpenModal = (e) => {
          e.preventDefault();
          dispatch(openModal())
          dispatch(startNewFormMode())
    }
      
    return (
        <>
            <main>
                <BreadCrumbs />
                <div>
                    <div className="fixed-action-btn">
                    <Link data-target="modal1" to="#" onClick={handleOpenModal} className="btn-floating btn-large red waves-effect waves-light btn modal-trigger">
                        <i className="large material-icons">add</i>
                    </Link>
                    </div>
                    <div 
                        
                        id="modal1"
                        className="modal"
                        >
                        <div className="modal-content">
                            {
                            <CrudLinesForm allStops={stops?.sort((a, b) => a.codeStop > b.codeStop ? 1 : -1)}/>
                            }
                        </div>
                        <div className="modal-footer flexForm">
                            {
                            //<Link to="#!" className="modal-close red darken-red-3 waves-effect waves-red btn" onClick={handlecloseModal}><i class="material-icons left">cancel</i>Cancelar</Link>
                            }
                        </div>
                    </div>
                </div>
                <div className=" animate__animated animate__fadeIn ">
                    <div className="row">
                        <div className="col offset-s3 s6">
                            <div className="card">
                            <table className=" striped responsive-table centered ">
                            <thead>
                                <tr>                            
                                    <th>Código</th>
                                    <th>Nombre</th>
                                    <th>Acciones</th>
                                </tr>
                            </thead>

                            <tbody>
                                
                                    {
                                    linesIsDownloaded && lines?.map(line => (
                                        <tr key={line.codeLine}>
                                            <td>{line.codeLine}</td>
                                            <td>{line.from} - {line.to} - {line.from}</td>
                                            <td>
                                                <Link to="!#" onClick={(e) => deleteLine(e,line)}>
                                                    <i className="material-icons red-text text-darken-4">delete</i>
                                                </Link>
                                                <Link data-target="modal1" to="!#" onClick={(e) => edit(e,line)} className="modal-trigger">
                                                    <i className="material-icons light-green-text text-darken-2">edit</i>
                                                </Link>
                                            </td>
                                        </tr>
                                    ))
                                    }
        
                                
                            </tbody>
                            </table>
                            </div>
                        </div>
                    </div>
                </div>
            </main>
        </>
    )
}
