import React, { useEffect } from 'react'
import { useDispatch, useSelector } from 'react-redux';
import M from "materialize-css/dist/js/materialize.min.js";
import "materialize-css/dist/css/materialize.min.css";
import { Link } from 'react-router-dom';
import { openModal } from '../../actions/modal';
import { startNewFormMode } from '../../actions/form';
import { allLinesLoading } from '../../actions/lines';
import { CrudLinesNotificationsForm } from '../ui/admin/CrudLinesNotificationsForm';
import { notificationsStartLoading, startDeletingNotification } from '../../actions/notification';
import { busesStartLoading } from '../../actions/buses';
import { CrudBusesNotificationsForm } from '../ui/admin/CrudBusesNotificationsForm';
import { BreadCrumbs } from '../ui/BreadCrumbs';
import Swal from 'sweetalert2';

export const CRUDNotificationScreen = () => {
        
    const dispatch = useDispatch();
    const {isDownloaded: linesIsDownloaded, data: lines} = useSelector(state => state.lines);
    const {isDownloaded: busesIsDownloaded, data: buses} = useSelector(state => state.buses); 
    const {isDownloaded: notificationsIsDownloaded, data: notifications} = useSelector(state => state.notifications); 

    useEffect(() => {        
        !linesIsDownloaded && dispatch(allLinesLoading())        
    }, [dispatch, linesIsDownloaded])
    
    useEffect(() => {        
        !busesIsDownloaded && dispatch((busesStartLoading()))        
    }, [dispatch, busesIsDownloaded])

    useEffect(() => {        
        //!notificationsIsDownloaded && dispatch(notificationsStartLoading())        
        dispatch(notificationsStartLoading());
        
    }, [dispatch/*, notificationsIsDownloaded*/])

    const deleteNotification = (e, notification) => {
        e.preventDefault();
        const swalWithBootstrapButtons = Swal.mixin({
            customClass: {
              confirmButton: 'btn btn-success ',
              cancelButton: 'btn btn-danger red mr10'
            },
            buttonsStyling: true
          })
          
          swalWithBootstrapButtons.fire({
            title: '¿Estás seguro?',
            text: "Este proceso es irreversible",
            icon: 'warning',
            showCancelButton: true,
            confirmButtonText: 'Sí, bórralo',
            cancelButtonText: '¡No, cancela!',
            reverseButtons: true
          }).then((result) => {
            if (result.isConfirmed) {
                dispatch(startDeletingNotification(notification))
                swalWithBootstrapButtons.fire(
                'Borrado',
                'El registro ha sido borrado.',
                'success'
              )
            } else if (
              result.dismiss === Swal.DismissReason.cancel
            ) {
              swalWithBootstrapButtons.fire(
                'Cancelado',
                'El proceso de borrado se ha cancelado',
                'error'
              )
            }
          })
    }
    
    useEffect(() => {
        let elems = document.querySelectorAll('.modal');
        M.Modal.init(elems, {
            inDuration: 250,
            outDuration: 250,
            opacity: 0.5,
            dismissible: false,
            startingTop: "4%",
            endingTop: "10%"
          });
          const floatButton = document.querySelectorAll('.fixed-action-btn');
          M.FloatingActionButton.init(floatButton, {
            direction: 'left',
            hoverEnabled: true
          });
      });
 
    const handleOpenModal = (e) => {
          e.preventDefault();
          dispatch(openModal())
          dispatch(startNewFormMode())
    }
      
    return (
        <>
            <main>  
                <BreadCrumbs />              
                <div>   
                    <div className="fixed-action-btn">
                        <div>
                            <Link onClick={e => e.preventDefault()} to="#" className="btn-floating btn-large red">
                                <i className="large material-icons">mode_edit</i>
                            </Link>
                            <ul>                                
                                <li><Link data-target="modal1" to="#" onClick={handleOpenModal} className="btn-floating yellow darken-1 btn modal-trigger"><i className="material-icons">linear_scale</i></Link></li>
                                <li><Link data-target="modal2" to="#" onClick={handleOpenModal} className="btn-floating blue btn modal-trigger"><i className="material-icons">directions_bus</i></Link></li>
                            </ul>
                        </div>
                    </div>
                    <div 
                        id="modal1"
                        className="modal"
                        >
                        <div className="modal-content">
                            {
                            <CrudLinesNotificationsForm allLines={lines} />
                            }
                        </div>
                        <div className="modal-footer flexForm">
                            {
                            //<Link to="#!" className="modal-close red darken-red-3 waves-effect waves-red btn" onClick={handlecloseModal}><i class="material-icons left">cancel</i>Cancelar</Link>
                            }
                        </div>
                    </div>
                    <div 
                        id="modal2"
                        className="modal"
                        >
                        <div className="modal-content">
                            {
                            <CrudBusesNotificationsForm allBuses={buses} />
                            }
                        </div>
                        <div className="modal-footer flexForm">
                            {
                            //<Link to="#!" className="modal-close red darken-red-3 waves-effect waves-red btn" onClick={handlecloseModal}><i class="material-icons left">cancel</i>Cancelar</Link>
                            }
                        </div>
                    </div>
                </div>
                <div className=" animate__animated animate__fadeIn ">
                    <div className="row">
                        <div className="col offset-s3 s6">
                            <div className="card">
                            <table className=" striped responsive-table centered ">
                            <thead>
                                <tr>                            
                                    <th>Número</th>
                                    <th>Identificación</th>
                                    <th>Acciones</th>
                                </tr>
                            </thead>
                            <tbody>                                
                                    {
                                    notificationsIsDownloaded && notifications?.map((notification, index) => (
                                        <tr key={notification._id}>
                                            <td>{index}</td>
                                            <td>{notification?.description}</td>                                            
                                            <td>
                                                <Link to="!#" onClick={(e) => deleteNotification(e,notification)}>
                                                    <i className="material-icons red-text text-darken-4">delete</i>
                                                </Link>
                                            </td>
                                        </tr>
                                    ))
                                    }
                            </tbody>
                            </table>
                            </div>
                        </div>
                    </div>
                </div>
            </main>
        </>
    )
}
