import React, { useEffect } from 'react'
import { useDispatch, useSelector } from 'react-redux';
import { startDeletingStop, stopsStartLoading } from '../../actions/stops';
import M from "materialize-css/dist/js/materialize.min.js";
import "materialize-css/dist/css/materialize.min.css";
import { Link } from 'react-router-dom';
import { CrudStopsForm } from '../ui/admin/CrudStopsForm';
import { openModal } from '../../actions/modal';
import { startEditFormMode, startNewFormMode } from '../../actions/form';
import { BreadCrumbs } from '../ui/BreadCrumbs';
import Swal from 'sweetalert2';

export const CRUDStopsScreen = () => {
        
    const dispatch = useDispatch();
    const {isDownloaded: stopsIsDownloaded, data: stops} = useSelector(state => state.stops); 
    
    useEffect(() => {        
        !stopsIsDownloaded && dispatch(stopsStartLoading())
    }, [dispatch, stopsIsDownloaded])

    const edit = (e, stop) => {
        e.preventDefault();
        dispatch(openModal(stop))
        dispatch(startEditFormMode())
    }

    const deleteStop = (e, stop) => {
        e.preventDefault();
        const swalWithBootstrapButtons = Swal.mixin({
            customClass: {
              confirmButton: 'btn btn-success ',
              cancelButton: 'btn btn-danger red mr10'
            },
            buttonsStyling: true
          })
          
          swalWithBootstrapButtons.fire({
            title: '¿Estás seguro?',
            text: "Este proceso es irreversible",
            icon: 'warning',
            showCancelButton: true,
            confirmButtonText: 'Sí, bórralo',
            cancelButtonText: '¡No, cancela!',
            reverseButtons: true
          }).then((result) => {
            if (result.isConfirmed) {
                dispatch(startDeletingStop(stop));
                swalWithBootstrapButtons.fire(
                'Borrado',
                'El registro ha sido borrado.',
                'success'
              )
            } else if (
              result.dismiss === Swal.DismissReason.cancel
            ) {
              swalWithBootstrapButtons.fire(
                'Cancelado',
                'El proceso de borrado se ha cancelado',
                'error'
              )
            }
          })
    }
    
    useEffect(() => {
        let elems = document.querySelectorAll('.modal');
        M.Modal.init(elems, {
            inDuration: 250,
            outDuration: 250,
            opacity: 0.5,
            dismissible: true,
            startingTop: "4%",
            endingTop: "10%"
          });
          const floatButton = document.querySelectorAll('.fixed-action-btn');
          M.FloatingActionButton.init(floatButton, {
            direction: 'left',
            hoverEnabled: false
          });
      });

    const handleOpenModal = (e) => {
          e.preventDefault();
          dispatch(openModal())
          dispatch(startNewFormMode())
    }
      
    return (
        <>
            <main>
                <BreadCrumbs />
                <div>
                    <div className="fixed-action-btn">
                    <Link data-target="modal1" to="#" onClick={handleOpenModal} className="btn-floating btn-large red waves-effect waves-light btn modal-trigger">
                        <i className="large material-icons">add</i>
                    </Link>
                    </div>
                    <div 
                        
                        id="modal1"
                        className="modal"
                        >
                        <div className="modal-content">
                            <CrudStopsForm />
                        </div>
                        <div className="modal-footer">
                            {
                            //<a href="#!" onClick={handlecloseModal} className="modal-close waves-effect waves-green btn-flat">Cerrar</a>
                            }
                        </div>
                    </div>
                </div>
                <div className=" animate__animated animate__fadeIn ">
                    <div className="row">
                        <div className="col offset-s3 s6">
                            <div className="card">
                            <table className="responsive-table striped centered">
                            <thead>
                                <tr>                            
                                    <th>Código</th>
                                    <th>Nombre</th>
                                    <th>Acciones</th>
                                </tr>
                            </thead>

                            <tbody>
                                
                                    {
                                    stopsIsDownloaded && stops?.map(stop => (
                                        <tr key={stop.codeStop}>
                                            <td>{stop.codeStop}</td>
                                            <td>{stop.name}</td>
                                            <td>
                                                <Link to="!#" onClick={(e) => deleteStop(e,stop)}>
                                                    <i className="material-icons red-text text-darken-4">delete</i>
                                                </Link>
                                                <Link data-target="modal1" to="#" onClick={(e) => edit(e,stop)} className="modal-trigger">
                                                    <i className="material-icons light-green-text text-darken-2">edit</i>
                                                </Link>
                                            </td>
                                        </tr>
                                    ))
                                    }
                            </tbody>
                            </table>
                            </div>
                        </div>
                    </div>
                </div>
            </main>
        </>
    )
}
