import React, { useEffect } from 'react'
import { useDispatch, useSelector } from 'react-redux';
import M from "materialize-css/dist/js/materialize.min.js";
import "materialize-css/dist/css/materialize.min.css";
import { Link } from 'react-router-dom';
import { openModal } from '../../actions/modal';
import { startEditFormMode } from '../../actions/form';
import { CrudUserForm } from '../ui/admin/CrudUserForm';
import { startDeletingUser, usersStartLoading } from '../../actions/users';
import { BreadCrumbs } from '../ui/BreadCrumbs';
import Swal from 'sweetalert2';

export const CRUDUserScreen = () => {
        
    const dispatch = useDispatch();
    const {isDownloaded: usersIsDownloaded, data: users} = useSelector(state => state.users); 

    useEffect(() => {        
        !usersIsDownloaded && dispatch(usersStartLoading())        
    }, [dispatch, usersIsDownloaded])


    const edit = (e, user) => {
        e.preventDefault();
        dispatch(openModal(user))
        dispatch(startEditFormMode())
    }

    const deleteUser = (e, user) => {
        e.preventDefault();
        const swalWithBootstrapButtons = Swal.mixin({
            customClass: {
              confirmButton: 'btn btn-success ',
              cancelButton: 'btn btn-danger red mr10'
            },
            buttonsStyling: true
          })
          
          swalWithBootstrapButtons.fire({
            title: '¿Estás seguro?',
            text: "Este proceso es irreversible",
            icon: 'warning',
            showCancelButton: true,
            confirmButtonText: 'Sí, bórralo',
            cancelButtonText: '¡No, cancela!',
            reverseButtons: true
          }).then((result) => {
            if (result.isConfirmed) {
                dispatch(startDeletingUser(user))
                swalWithBootstrapButtons.fire(
                'Borrado',
                'El registro ha sido borrado.',
                'success'
              )
            } else if (
              result.dismiss === Swal.DismissReason.cancel
            ) {
              swalWithBootstrapButtons.fire(
                'Cancelado',
                'El proceso de borrado se ha cancelado',
                'error'
              )
            }
          })        
    }

    
    useEffect(() => {
        let elems = document.querySelectorAll('.modal');
        M.Modal.init(elems, {
            inDuration: 250,
            outDuration: 250,
            opacity: 0.5,
            dismissible: false,
            startingTop: "4%",
            endingTop: "10%"
          });
          const floatButton = document.querySelectorAll('.fixed-action-btn');
          M.FloatingActionButton.init(floatButton, {
            direction: 'left',
            hoverEnabled: false
          });
      });
     
    return (
        <>
            <main>
                <BreadCrumbs />
                <div>
                    <div className="fixed-action-btn">
                    {
                    /* EL ADMIN NO PUEDE AÑADIR USUARIOS
                    <Link data-target="modal1" to="#" onClick={handleOpenModal} className="btn-floating btn-large red waves-effect waves-light btn modal-trigger">
                        <i className="large material-icons">add</i>
                    </Link>
                    */
                    }
                    </div>
                    <div 
                        
                        id="modal1"
                        className="modal"
                        >
                        <div className="modal-content">
                            {
                            <CrudUserForm />
                            }
                        </div>
                        <div className="modal-footer">
                            {
                            //<a href="#!" className="modal-close waves-effect waves-green btn-flat" onClick={handlecloseModal}>Cerrar</a>
                            }
                        </div>
                    </div>
                </div>
                <div className=" animate__animated animate__fadeIn ">
                    <div className="row">
                        <div className="col offset-s3 s6">
                            <div className="card">
                            <table className=" striped responsive-table centered ">
                            <thead>
                                <tr>                            
                                    <th>Número</th>
                                    <th>Nombre</th>
                                    <th>Acciones</th>
                                </tr>
                            </thead>
                            <tbody>                                
                                    {
                                    usersIsDownloaded && users?.map((user, index) => (
                                        <tr key={user._id}>
                                            <td>{index}</td>
                                            <td>{user?.name}</td>                                            
                                            <td>
                                                <Link to="!#" onClick={(e) => deleteUser(e,user)}>
                                                    <i className="material-icons red-text text-darken-4">delete</i>
                                                </Link>
                                                <Link data-target="modal1" to="!#" onClick={(e) => edit(e,user)} className="modal-trigger">
                                                    <i className="material-icons light-green-text text-darken-2">edit</i>
                                                </Link>
                                            </td>
                                        </tr>
                                    ))
                                    }
                            </tbody>
                            </table>
                            </div>
                        </div>
                    </div>
                </div>
            </main>
        </>
    )
}
