import React, {useEffect, useRef, useState} from 'react';
import { useDispatch } from 'react-redux';
import { startLogin, startRegister } from '../../actions/auth';

import M from "materialize-css/dist/js/materialize.min.js";
import "materialize-css/dist/css/materialize.min.css";
import { useForm } from '../../hooks/useForm';
//import { Link } from 'react-router-dom';

export const LoginScreen = () => {

    const dispatch = useDispatch();

    useEffect(() => {
        M.AutoInit();      
    })

    const fixedLoginHeader = useRef()
    const login = useRef(null)

    useEffect(() => {
        const showScrollTop = () =>{
            const currentScrollPosition = window.pageYOffset; 
            if(currentScrollPosition > login.current?.offsetTop -500){
                fixedLoginHeader.current?.className && (fixedLoginHeader.current.className="fixedLoginHeader fixedLoginHeaderShow animate__animated animate__fadeInDown z-depth-2")
            } else {
                fixedLoginHeader.current?.className && (fixedLoginHeader.current.className="fixedLoginHeader animate__animated animate__fadeOutUp z-depth-2")
            }
          }
        
          window.addEventListener('scroll', showScrollTop)

          return () => window.removeEventListener("scroll", showScrollTop, false);
        
    }, [])

    const [ formLoginValues, handleLoginInputChange ] = useForm({
        lEmail: '',
        lPassword: ''
    });

    const [ formRegisterValues, handleRegisterInputChange ] = useForm({
        rName: '',
        rSurname: '',
        rEmail1: '',
        rEmail2: '',
        rPassword1: '',
        rPassword2: ''
    });

    const { lEmail, lPassword } = formLoginValues;
    const { rName, rSurname, rEmail1, rEmail2, rPassword1, rPassword2 } = formRegisterValues;

    const [errMsg, setErrMsg] = useState('');
    const getErrors = (errorMsg) =>{
        setErrMsg(errorMsg);
    }

    const handleLogin = ( e ) => {
        e.preventDefault();
        dispatch( startLogin( lEmail, lPassword, getErrors ) );        
    }

    const handleRegister = ( e ) => {
        e.preventDefault();

        if ( rPassword1 !== rPassword2 ) {
            //return Swal.fire('Error', 'Las contraseñas deben de ser iguales','error');
        }
        console.log('?')
        dispatch( startRegister( rEmail1, rPassword1, rName, rSurname ) );
    }

    const handleScroll = () => {
        login.current.scrollIntoView({ behavior: "smooth", block: "start"});
    }
    


    return (
        <>
        <main className="p0">
            <div ref={fixedLoginHeader} className="fixedLoginHeader fixedLoginHeaderHide">
                <h1><img alt="Bahía Bus Logo" src="/images/logo.svg"></img></h1>
            </div>
            <section id="index-banner" className="row banner-container">
                <div className="height100 valign-wrapper">
                    <div className="container">
                    <img src="/images/logo.svg" className="loginLogo" alt="Logo de Bahía Bus" />
                    <h1 className="header right-align orange-text text-darken-2" style={{margin: 0}}>BAHÍA-BUS</h1>
                    <div className="row center">
                        <h5 className="header-para col s12 black-text text-darken-3 right-align">Tu web de movilidad en Sanlúcar de Barrameda.</h5>
                    </div>
                    <div className="row right">
                        <button onClick={handleScroll} className="btn-large waves-effect waves-light orange darken-4 modal-trigger ">Regístrate</button>
                    </div>
                    <br /><br />
                    </div>
                </div>
            </section>
            <section className="col-description valign-wrapper">
                <div className="container">
                    <div className="section">
                        <div className="row">
                            <article className="col s12 l4">
                                <div className="icon-block">
                                <h2 className="center white-text"><i className="material-icons">place</i></h2>
                                <h5 className="center">Encuentra un destino</h5>
                                <p className="light">Si necesitas conocer qué línea de autobús te lleva desde un punto a otro de la ciudad, ahora puedes hacerlo a través del buscador de la aplicación.</p>
                                </div>
                            </article>
                            <article className="col s12 l4">
                                <div className="icon-block">
                                <h2 className="center white-text"><i className="material-icons">alarm</i></h2>
                                <h5 className="center">Seguimiento en tiempo real</h5>
                                <p className="light">Una solución digital que dota a los viajeros de un mayor control informativo en tiempo real acerca de la ubicación actual de los autobuses urbanos.</p>
                                </div>
                            </article>

                            <article className="col s12 l4">
                                <div className="icon-block">
                                <h2 className="center white-text"><i className="material-icons">notifications</i></h2>
                                <h5 className="center">Recibe notificaciones</h5>
                                <p className="light">Recibe las notificaciones de las líneas de autobús, u otras incidencias como posibles retrasos o paradas anuladas. En definitiva, siente que tu sistema de transporte es eficiente.</p>
                                </div>
                            </article>
                        </div>
                    </div>
                </div>
            </section>
            <section className="loginBack">
                <div className="loginContainer container white z-depth-2">
                    <ul className="tabs orange darken-4">
                        <li className="tab col s3"><a className="white-text active" href="#login">login</a></li>
                        <li className="tab col s3"><a className="white-text" href="#register">registrarse</a></li>
                    </ul>
                    <div ref={login} id="login" className="col s12">
                        <form className="col s12" onSubmit={ handleLogin }>
                        <div className="form-container">
                            <h3 className="orange-text">Login</h3>
                            <div className="row">
                            <div className="input-field col s12">
                                <i className="material-icons prefix">account_circle</i>
                                <input id="email" type="email" autoComplete="username" className="validate" name="lEmail" value={ lEmail } onChange={ handleLoginInputChange } />                        
                                <label className="active" htmlFor="email">Email</label>
                            </div>
                            </div>
                            <div className="row">
                            <div className="input-field col s12">
                                <i className="material-icons prefix">vpn_key</i>
                                <input id="password" autoComplete="current-password" type="password" className="validate" name="lPassword" value={ lPassword } onChange={ handleLoginInputChange } />
                                <label className="active" htmlFor="password">Password</label>
                            </div>
                            </div>
                            <br />
                            <div className="center-align">
                                <button className="btn waves-effect waves-light orange darken-4" type="submit" name="action">Conectar</button>
                                {
                                /*
                                AÚN NO ESTÁ IMPLEMENTADO
                                <br />
                                <br />
                                
                                <Link className="orange-text darken-4"to="#!">¿Olvidaste la contraseña?</Link>
                                */
                                }
                                
                                <div className="helper-text" data-error="wrong">{errMsg}</div>
                            </div>
                        </div>
                        </form>
                    </div>
                    <div id="register" className="col s12">
                        <form className="col s12" onSubmit={ handleRegister }>
                            <div className="form-container">
                                <h3 className="orange-text">Regístrate</h3>
                                <div className="row">
                                <div className="input-field col s6">
                                    <input id="first_name" type="text" className="validate"  name="rName" value={ rName } onChange={ handleRegisterInputChange } />
                                    <label className="active" htmlFor="first_name">Nombre</label>
                                </div>
                                <div className="input-field col s6">
                                    <input id="last_name" type="text" className="validate" name="rSurname" value={ rSurname } onChange={ handleRegisterInputChange } />
                                    <label className="active" htmlFor="last_name">Apellidos</label>
                                </div>
                            </div>
                            <div className="row">
                                <div className="input-field col s12">
                                    <input id="remail" autoComplete="username" type="email" className="validate"  name="rEmail1" value={ rEmail1 } onChange={ handleRegisterInputChange } />
                                    <label className="active" htmlFor="remail">Email</label>
                                </div>
                                </div>
                                <div className="row">
                                    <div className="input-field col s12">
                                        <input id="email-confirm" autoComplete="username" type="email" className="validate" name="rEmail2" value={ rEmail2 } onChange={ handleRegisterInputChange }  />
                                        <label className="active" htmlFor="email-confirm">Email Confirmación</label>
                                    </div>
                                </div>
                                <div className="row">
                                    <div className="input-field col s12">
                                        <input id="password2" autoComplete="new-password" type="password" className="validate" name="rPassword1" value={ rPassword1 } onChange={ handleRegisterInputChange }   />
                                        <label className="active" htmlFor="password2">Password</label>
                                    </div>
                                </div>
                                <div className="row">
                                    <div className="input-field col s12">
                                        <input id="password-confirm" autoComplete="new-password" type="password" className="validate" name="rPassword2" value={ rPassword2 } onChange={ handleRegisterInputChange }   />
                                        <label className="active" htmlFor="password-confirm">Password Confirmación</label>
                                    </div>
                                </div>
                                <center>
                                <button className="btn waves-effect waves-light orange darken-4" type="submit" name="action">Enviar</button>
                                </center>
                            </div>
                        </form>
                    </div>
                </div>
            </section>
            </main>
            <footer className="page-footer orange darken-4">
            <div className="footer-copyright orange darken-4">
                <div className="container orange darken-4">
                © 2022 Copyright Cristian González Vélez
                </div>
            </div>
            </footer>
</>
    )
}
