import React, { useEffect, useRef, useState } from 'react'
import { useDispatch, useSelector } from 'react-redux';
import { Link } from 'react-router-dom';
import { busesStartLoading } from '../../actions/buses';
import { startNewFormMode } from '../../actions/form';
import { openModal } from '../../actions/modal';
import { useSocket } from '../../hooks/useSocket';
import { CrudBusesNotificationsForm } from '../ui/admin/CrudBusesNotificationsForm';
import M from "materialize-css/dist/js/materialize.min.js";
import "materialize-css/dist/css/materialize.min.css";
import { BreadCrumbs } from '../ui/BreadCrumbs';
import { linea1A, linea1B, linea2, linea3, linea4 } from '../../demo/coordinates';
import { useForm } from '../../hooks/useForm';
import Swal from 'sweetalert2';

const baseUrl = process.env.REACT_APP_SOCKET_URL;

export const BusScreen = () => {
    const selectBus = useRef();
    const ownCoordsSwitch = useRef();    
    const playButton = useRef();
    const dispatch = useDispatch();
    const {isDownloaded: busesIsDownloaded, data:buses} = useSelector(state => state.buses);
    const { socket, online, conectarSocket, desconectarSocket } = useSocket(`${baseUrl}`);
    
    // Inicialización de formulario
    const [ formBusValues, handleBusInputChange ] = useForm({
        identification : '',
        ownCoords: false,
    });

    const {
        identification,
        ownCoords,
    } = formBusValues;

    useEffect(() => {        
        !busesIsDownloaded && dispatch((busesStartLoading()));
    }, [dispatch, busesIsDownloaded])

    // Cargamos materializecss
    useEffect(() => {
        M.FormSelect.init(selectBus.current, {});
    })
   const intervalsArray = useRef([]);
   const GPSInitialState ={latitud: -1, logintud: -1};
   const [coordenadasGPS, setCoordenadasGPS] = useState(GPSInitialState);
   useEffect(() => {
            // Conexión con el socket server
            
            if(online){
                intervalsArray.current.map(a => clearInterval(a));
                intervalsArray.current = [];

                if(ownCoordsSwitch.current.checked){
                    setTimeout(()=>{
                        socket.emit('marcador-nuevo', {id: identification, lng: -6.3397 , lat: 36.7751, line:  buses?.find(bus => bus?.identification === identification)?.codeLine });
                    }, 2000)
                    intervalsArray.current.push(setInterval(async () => {
                        new Promise((resolve, reject) => 
                            navigator.geolocation.getCurrentPosition(resolve, reject, {
                                enableHighAccuracy: true,
                                timeout: 10000,
                                maximumAge: 600000
                              })
                        ).then(pos => {
                            console.log(pos.coords.longitude, pos.coords.latitude, new Date());
                            setCoordenadasGPS({longitud : pos.coords.longitude, latitud: pos.coords.latitude} );
                            socket.emit('marcador-actualizado', {id: identification, lng: pos.coords.longitude , lat: pos.coords.latitude, line: buses?.find(bus => bus?.identification === identification)?.codeLine  });                            
                        });
                    }, 10000));
                }
                else{
                    //Línea 4. Bus 1.
                    if(identification === 'L4-1' || identification === 'demo'){
                        // TO-DO: Si no espera un poco, no emite la primera vez. 
                        // Pero el caso es que está ready. Hay que investigar por qué.
                        setTimeout(()=>{
                            socket.emit('marcador-nuevo', {id: "L4-1", lng: -6.3397 , lat: 36.7751, line: 4  });
                            setTimeout(()=>{
                                let index = 0;
                                intervalsArray.current.push(setInterval(() => {                
                                        let lng = linea4[(index++) % linea4.length][0];
                                        let lat = linea4[(index++) % linea4.length][1];
                                        socket.emit('marcador-actualizado', {id: "L4-1", lng: lng , lat: lat, line: 4  });
                                }, 1000));
                            }, 1000)
                        }, 2000) 
                    }
            
                    //Línea 4. Bus 2.
                    if(identification === 'L4-2' || identification === 'demo'){
                        setTimeout(()=>{
                            socket.emit('marcador-nuevo', {id: "L4-2", lng: -6.3397 , lat: 36.7751, line: 4  });
                            setTimeout(()=>{
                                let index2 = 2000;        
                                intervalsArray.current.push(setInterval(() => {                
                                    let lng = linea4[(index2++) % linea4.length][0];
                                    let lat = linea4[(index2++) % linea4.length][1];
                                    socket.emit('marcador-actualizado', {id: "L4-2", lng: lng , lat: lat, line: 4  });
                                }, 1000));
                            }, 1000)

                        }, 2000)
                        
                    }
            
                    //Línea 3. Bus 3.
                    if(identification === 'L3-1' || identification === 'demo'){
                        setTimeout(()=>{
                            socket.emit('marcador-nuevo', {id: "L3-1", lng: -6.3397 , lat: 36.7751, line: '3'  });
                            setTimeout(()=>{
                                let index3 = 0;
                                intervalsArray.current.push(setInterval(() => {
                                        let lng = linea3[(index3++) % linea3.length][0];
                                        let lat = linea3[(index3++) % linea3.length][1];
                                        socket.emit('marcador-actualizado', {id: "L3-1", lng: lng , lat: lat, line: '3'  });
                                }, 1000));
                            }, 1000)
                        }, 2000)
                        
                    }
            
                    //Línea 3. Bus 4.
                    if(identification === 'L3-2' || identification === 'demo'){
                        setTimeout(()=>{
                            socket.emit('marcador-nuevo', {id: "L3-2", lng: -6.3397 , lat: 36.7751, line: '3'  });
                            setTimeout(()=>{
                                let index4 = 2000;
                                intervalsArray.current.push(setInterval(() => {                
                                        let lng = linea3[(index4++) % linea3.length][0];
                                        let lat = linea3[(index4++) % linea3.length][1];
                                        socket.emit('marcador-actualizado', {id: "L3-2", lng: lng , lat: lat, line: '3'  });
                                }, 1000));
                            }, 1000)
                        }, 2000)                        
                    }
            
                    
                    //Línea 1A. Bus 1.
                    if(identification === 'L1-1' || identification === 'demo'){
                        setTimeout(()=>{
                            socket.emit('marcador-nuevo', {id: "L1-1", lng: -6.3397 , lat: 36.7751, line: '1'  });
                            setTimeout(()=>{
                                let index5 = 0;
                                intervalsArray.current.push(setInterval(() => {
                                        let lng = linea1A[(index5++) % linea1A.length][0];
                                        let lat = linea1A[(index5++) % linea1A.length][1];
                                        socket.emit('marcador-actualizado', {id: "L1-1", lng: lng , lat: lat, line: '1'  });
                                }, 1000));
                            }, 1000)
                        }, 2000)                        
                    }
            
                    //Línea 1B. Bus 1.
                    if(identification === 'L1-2' || identification === 'demo'){
                        setTimeout(()=>{
                            socket.emit('marcador-nuevo', {id: "L1-2", lng: -6.3397 , lat: 36.7751, line: '1'  });
                            setTimeout(()=>{
                                let index6 = 0;        
                                intervalsArray.current.push(setInterval(() => {                
                                    let lng = linea1B[(index6++) % linea1B.length][0];
                                    let lat = linea1B[(index6++) % linea1B.length][1];
                                    socket.emit('marcador-actualizado', {id: "L1-2", lng: lng , lat: lat, line: '1'  });
                                }, 1000))
                            }, 1000)    
                        }, 2000)
                        
                    }
                    
                    //Línea 2. Bus 1.                
                    if(identification === "L2-1" || identification === 'demo'){
                        setTimeout(()=>{
                            socket.emit('marcador-nuevo', {id: "L2-1", lng: -6.3397 , lat: 36.7751, line: '2'  });
                            setTimeout(()=>{
                                let index7 = 0;
                                intervalsArray.current.push(setInterval(() => {                
                                    let lng = linea2[(index7++) % linea2.length][0];
                                    let lat = linea2[(index7++) % linea2.length][1];
                                    socket.emit('marcador-actualizado', {id: "L2-1", lng: lng , lat: lat, line: '2'  });
                                }, 1000));
                            }, 1000)
                        }, 2000)                        
                    }
            
                    //Línea 2. Bus 2.
                    if(identification === 'L2-2' || identification === 'demo'){
                        setTimeout(()=>{
                            socket.emit('marcador-nuevo', {id: "L2-2", lng: -6.3397 , lat: 36.7751, line: '2'  });
                            setTimeout(()=>{
                                let index8 = 2000;        
                                intervalsArray.current.push(setInterval(() => {                
                                    let lng = linea2[(index8++) % linea2.length][0];
                                    let lat = linea2[(index8++) % linea2.length][1];
                                    socket.emit('marcador-actualizado', {id: "L2-2", lng: lng , lat: lat, line: '2'  });
                                }, 1000));
                            }, 1000)
                        }, 2000)
                        
                    }
                }
            }
    }, [socket, online, identification, ownCoords, buses])

    useEffect(() => {
        let elems = document.querySelectorAll('.modal');
        M.Modal.init(elems, {
            inDuration: 250,
            outDuration: 250,
            opacity: 0.5,
            dismissible: false,
            startingTop: "4%",
            endingTop: "10%"
          });
          const floatButton = document.querySelectorAll('.fixed-action-btn');
          M.FloatingActionButton.init(floatButton, {
            direction: 'left',
            hoverEnabled: false
          });
      });


    const handleOpenModal = (e) => {
        e.preventDefault();
        dispatch(openModal())
        dispatch(startNewFormMode())
  }
  
  const clearAllIntervals = () => {
        intervalsArray.current.map(a => clearInterval(a));
        intervalsArray.current = [];
        socket?.emit('marcador-borrado', identification);
        playButton.current.className='btn waves-effect waves-light teal modal-close';
        selectBus.current.disabled = false;
        ownCoordsSwitch.current.disabled = false;
        setCoordenadasGPS(GPSInitialState);
  }

  useEffect(() => {
    socket?.on('marcador-borrado', (identification)=>{
        desconectarSocket();
        console.log("Desconecto!!");
    })
    return ()=> socket?.off("marcador-borrado");           

}, [socket, desconectarSocket])

const handleConnection = () => {
    if(ownCoordsSwitch.current.checked && identification === 'demo'){
        Swal.fire("Solo a un autobús", "Solo puedes enviar coordenadas si eliges un solo autobús.", "error");
        return;
    }
    else if(identification){

        conectarSocket(identification);
        selectBus.current.disabled = true;
        ownCoordsSwitch.current.disabled = true;
        playButton.current.className='btn waves-effect waves-light teal modal-close disabled';
        setCoordenadasGPS(GPSInitialState);
    }
    else{
        Swal.fire("Falta el bus", "Elige un autobús antes de iniciar", "error");
    }
    
  }
    return (
        <main>
            <BreadCrumbs />
            <div>
                <div className="row">
                    <div className="col col s12 xl6 push-xl3">
                    <div className="card white darken-1">
                        <div className="card-content black-text">
                        <span className="card-title">{online ? 'Demo iniciada' : 'Demo no iniciada'}</span>
                            <form>
                                <div className="row">
                                    <div className="input-field col s12">
                                        <div className="switch"> 
                                            <label>
                                            <small>(No hace falta enviar nuestra ubicación si lo que se quiere es simular)</small><br/>
                                            Enviar mi ubicación por GPS
                                            
                                            <input type="checkbox" name="sendCoords" ref={ownCoordsSwitch}/>
                                            <span className="lever"></span>
                                            </label>
                                            {
                                            (coordenadasGPS.latitud !== -1 && coordenadasGPS.longitud !== -1) &&
                                            <div>
                                            <p><small>Latitud: <strong>{coordenadasGPS.latitud}</strong></small></p>
                                            <p><small>Longitud: <strong>{coordenadasGPS.longitud}</strong></small></p>
                                            </div>
                                            }
                                        </div>
                                    </div>
                                </div>
                                <div className="row">
                                    <div className="input-field col s12">
                                        <div>
                                            <i className="material-icons prefix">directions_bus</i>
                                            <select ref={selectBus} onChange={ handleBusInputChange } name="identification" >
                                                <option value='' defaultValue=''>Elige Bus</option>
                                                {
                                                    buses.map(bus => (<option key={bus._id} value={bus.identification}>{bus.identification} </option> ))
                                                }                           
                                                <option value='demo' defaultValue='demo'>Todos</option>
                                            </select>
                                            <label className="active" htmlFor="from">Identificación</label>
                                        </div>
                                    </div>
                                </div>  
                            </form>
                        </div>
                        <div className="card-action">
                            <button ref={playButton} className="btn waves-effect waves-light teal modal-close" onClick={handleConnection}>
                                <i className="material-icons white-text text-darken-4 left ">play_arrow</i>Iniciar
                            </button>
                            <button to="#!" className="ml15 modal-close red darken-red-3 waves-effect waves-red btn" onClick={clearAllIntervals} >
                                <i className="material-icons left">stop</i>Detener
                            </button>
                        </div>
                    </div>
                    </div>
                </div>

                <div className="fixed-action-btn">
                    <Link data-target="modal1" to="#" onClick={handleOpenModal} className="btn-floating btn-large red waves-effect waves-light btn modal-trigger">
                        <i className="large material-icons">add</i>
                    </Link>
                </div>
                <div 
                        id="modal1"
                        className="modal"
                        >
                        <div className="modal-content">
                            {
                            <CrudBusesNotificationsForm allBuses={buses} />
                            }
                        </div>
                        <div className="modal-footer flexForm">
                        </div>
                </div>            
            </div>
        </main>
    )
}