import React, { useEffect } from 'react';
import { BreadCrumbs } from '../ui/BreadCrumbs';
import { useDispatch, useSelector } from 'react-redux';
import { allLinesLoading } from '../../actions/lines';
import { MapLines } from '../ui/MapLines';
import Preloader from '../ui/Preloader';
import { ListLines } from '../ui/ListLines';
import { startChecking } from '../../actions/auth';

export const LinesScreen = () => {
    const dispatch = useDispatch();
    const lines = useSelector(state => state.lines)
    const {uid} = useSelector(state => state.auth)
    useEffect(() => {   
        dispatch(startChecking(uid))
    }, [dispatch, uid])

    useEffect(() => { 
        if(!lines.isDownloaded){
            dispatch(allLinesLoading())
        }
        
    }, [dispatch, lines])
    
    if(!lines.isDownloaded){
        return (
            <main>
            <BreadCrumbs />
            <Preloader />
            </main>)
    }
    else 
    return (
        <>
        <main>
            <BreadCrumbs />

            <div className="mb0">                        
                <div className="row mb0">
                    <div className="input-field col s12 mb0">
                        <h5>Líneas disponibles en Sanlúcar</h5>
                    </div>
                </div>
            </div>
            <div className="mt10 pAll10 animate__animated animate__fadeIn">
                <div className="row">
                    <div className="col s12 m12 pAll0">
                        <div className="white black-text pAll0 animate__animated animate__fadeInDown">
                            <MapLines  lineas={lines.data.map(line => {
                              return {
                                  path: line?.route?.path || [],
                                  color: line?.colorStart || '',
                                  desc: line?.codeLine + " - " + line.from + " - " + line.to + " - " + line.from
                                }
                            })} />                            
                        </div>
                    </div>                
                </div>
                <div className="row">
                    <ListLines lines={lines.data} />
                </div>
            </div>
        </main>
        </>

    )
}
