import React from 'react'
import { BreadCrumbs } from '../ui/BreadCrumbs'
import { useForm } from '../../hooks/useForm';
import { useDispatch } from 'react-redux';
import Swal from 'sweetalert2';
import { changeProfile } from '../../actions/profile';


export const ProfileScreen = () => {   
    const dispatch = useDispatch();

    const [ formProfileValues, handleProfileInputChange, resetValues ] = useForm({
        password1: '',
        password2: ''
    });

    const {password1, password2} = formProfileValues;

    const handleSubmit = (e)=> {
        e.preventDefault();
        if (password1 !== password2){
            Swal.fire('¿Has escrito bien las passwords?', '¡Creo que no!', 'question')
        }
        else if (!password1.length || !password2.length){
            Swal.fire('¿Has rellenado ambas passwords?', '¡Creo que no!', 'question')
        }
        else {
            dispatch (changeProfile({password1}))
            resetValues();
        }
        
        
    }
    
    return (
        <main>
            <BreadCrumbs />
            <div className="mt10 pAll10 animate__animated animate__bounceInUp">
                <div className="row">                
                    <section className="col s12 m12 l12 x12">
                        <div className="card ">
                            <div className="card-content p0">
                                <header className="card-title white-text generalCardHeader center-align card-title-text">
                                    <p>Modificar Contraseña de Usuario</p> 
                                    <i className="small material-icons">person</i>
                                </header>
                                <main className="white black-text pl20 pr20 pt20 pb20">
                                    <div id="login" className="">
                                        <form className="" onSubmit={ handleSubmit }>
                                        <div className="form-container">
                                            <div className="row">
                                            <div className="input-field col s12">
                                                <i className="material-icons prefix">vpn_key</i>
                                                <input id="password1" autoComplete="current-password" type="password" className="validate" name="password1" value={ password1 } onChange={ handleProfileInputChange } />

                                                <label htmlFor="password1">Password</label>
                                            </div>
                                            </div>
                                            <div className="row">
                                            <div className="input-field col s12">
                                                <i className="material-icons prefix">vpn_key</i>
                                                <input id="password2" autoComplete="current-password" type="password" className="validate" name="password2" value={ password2 } onChange={ handleProfileInputChange } />
                                                <label htmlFor="password2">Confirmar Password</label>
                                            </div>
                                            </div>
                                            <br />
                                            <div className="center-align">
                                                <button className="btn waves-effect waves-light teal" type="submit" name="action">Modificar</button>
                                                
                                                <div className="helper-text" data-error="wrong">{ /*errMsg */}</div>
                                            </div>
                                        </div>
                                        </form>
                                    </div>
                                </main>                                        
                            </div>
                        </div> 
                    </section>            
                </div>
            </div>                
        </main>
    )
}
