import { BreadCrumbs } from '../ui/BreadCrumbs';
import { useSelector, useDispatch} from 'react-redux';
import { SearchPathForm } from '../ui/SearchPathForm';
import { StopsPathFound } from '../ui/StopsPathFound';
import { WalkPath } from '../ui/WalkPath';
import { MapUI } from '../ui/MapUI';
import '../../css/styles.css';
import { useCallback, useEffect,useRef,useState} from 'react';
import { getCurrentPosition } from '../../helpers/currentPosition';
import { walkingPathDestinationStartLoading, walkingPathOriginStartLoading, walkingPathResetDestination } from '../../actions/walkingPath';

export const SearchResultsScreen = () => {
    const search = useSelector(state => state.search);
    const jsonWalkingPath = useSelector(state => state.walkingPath);
    const nearestToDestination = useSelector(state => state.nearestStop);
    const dispatch = useDispatch();
        
    useEffect(() => {
        
        if(search.length > 0){
            const endCoords = [search[0]?.longitude, search[0]?.latitude];            
            const getSteps = async() => {
                const startCoords = await getCurrentPosition()
                
                dispatch(walkingPathOriginStartLoading([startCoords.coords.longitude, startCoords.coords.latitude], endCoords))        
            }
            getSteps();
        }
    }, [search, dispatch])

    useEffect(() => {
        if(!nearestToDestination?.Destination?.Coordenadas?.length){
            dispatch(walkingPathResetDestination());            
        }
        if(search.length > 0 && nearestToDestination?.Destination?.Coordenadas?.length){
            const startCoords = [search[search.length-1]?.longitude, search[search.length-1]?.latitude];            
            const endCoords = nearestToDestination?.Destination?.Coordenadas;                         
            dispatch(walkingPathDestinationStartLoading(startCoords, endCoords));            
        }

    }, [search, dispatch, nearestToDestination])

    const topMapa = useRef()
    const [foco, setFoco] = useState({
        lng: '',
        lat: '',
        name: ''
    })

    const handleClick = useCallback((lngLat) => {
            
        const endCoords = [lngLat.lng, lngLat.lat];            
        const getSteps = async() => {
            const currentPosition = await getCurrentPosition()            
            const startCoords = [currentPosition.coords.longitude, currentPosition.coords.latitude]
            dispatch(walkingPathOriginStartLoading(startCoords, endCoords)) 
            topMapa.current.scrollIntoView({ behavior: "smooth", block: "start"});
            setFoco(lngLat)       
        }
        getSteps();
        

    },[dispatch])

    return (
        <main>
            <BreadCrumbs />
            <SearchPathForm />
            <div className="mt0 ">
                <div className="row">                        
                    <div className="col s12 m12 pAll0">
                        <div className="white black-text pAll0" ref={topMapa}>           
                           {
                            <MapUI focus={foco} paradas={search}/>               
                           }
                        </div>
                    </div>
                </div>
            </div>
            <div className="mt10 pAll10">
                <div className="row">                                            
                    <section className="col s12 m12 l12 xl6">
                        <div className="card">
                            <div className="card-content p0">
                                <header className="card-title white-text line1 center-align card-title-text">
                                    <p className='tituloResultados'>Itinerario desde {
                                       //Origen
                                       search[0]?.name
                                    } hasta {
                                       // Destino
                                       search[search?.length-1]?.name

                                    } </p> <i className="small material-icons">timeline</i>
                                </header>
                                <main className="white black-text pl20 pr20 pt20 pb20">           
                                    <StopsPathFound search={search} handleClick={handleClick}/>                
                                </main>
                            </div>
                        </div>
                    </section>                    
                    <section className="col s12 m12 l12 xl6">
                        <div className="card">
                            <div className="card-content p0">
                                <header className="card-title white-text line1 center-align card-title-text">
                                    <p className='tituloResultados'>Ruta a pie hasta {
                                        // Origen
                                        foco.name || search[0]?.name
                                    } </p> <i className="small material-icons">directions_walk</i>
                                </header>
                                <main className="white black-text pl20 pr20 pt20 pb20">
                                    { 
                                    
                                    jsonWalkingPath && <WalkPath 
                                                                toOriginSteps={jsonWalkingPath?.Origen?.routes[0]?.legs[0]?.steps} 
                                                                durationStepsToOrigin={Math.floor(jsonWalkingPath?.Origen?.routes[0]?.duration / 60)} 
                                                                toDestinationSteps={jsonWalkingPath?.Destino?.routes[0].legs[0].steps} 
                                                                durationStepsToDestination={Math.floor(jsonWalkingPath?.Destino?.routes[0].duration / 60)} 
                                                        />
                                    }
                                </main>
                            </div>
                        </div>
                    </section>
                </div>
            </div>
        </main>
    )
}
