import React, { useCallback, useEffect, useRef, useState } from 'react';
import { BreadCrumbs } from '../ui/BreadCrumbs';
import { useDispatch, useSelector } from 'react-redux';
import { MapLines } from '../ui/MapLines';
import Preloader from '../ui/Preloader';
import { stopsStartLoading } from '../../actions/stops';
import { getCurrentPosition } from '../../helpers/currentPosition';
import { walkingPathOriginStartLoading, walkingPathResetOrigin } from '../../actions/walkingPath';
import { StopsLine } from '../ui/StopsLine';
import { WalkPath } from '../ui/WalkPath';
import { useForm } from '../../hooks/useForm';

export const StopsScreen = () => {
    const dispatch = useDispatch();
    const stops = useSelector(state => state.stops)
    const titulo = useRef()
    const [foco, setFoco] = useState({
        lng: '',
        lat: '',
        name: '-selecciona parada-'
    })
    const jsonWalkingPath = useSelector(state => state.walkingPath)
    const [ stopSearchValue, handleSearchInputChange ] = useForm({
        stopName: '',
    });
    const { stopName } = stopSearchValue;

    const [stopsFiltered, setStopsFiltered] = useState([])

    
    useEffect(() => {            
            setStopsFiltered(stops.data.filter(stop => stop.name.toLowerCase().includes(stopName.toLowerCase())))
    }, [stopName, stops.data])



    useEffect(() => {

        if(!stops.isDownloaded){
            dispatch(stopsStartLoading())
           
        }
        dispatch(walkingPathResetOrigin());
        
    }, [dispatch, stops])

    const handleClick = useCallback((lngLat) => {
            
        const endCoords = [lngLat.lng, lngLat.lat];            
        const getSteps = async() => {
            const currentPosition = await getCurrentPosition();
            const startCoords = [currentPosition.coords.longitude, currentPosition.coords.latitude];
            dispatch(walkingPathOriginStartLoading(startCoords, endCoords));
            titulo.current.scrollIntoView({ behavior: "smooth", block: "start"});
            setFoco(lngLat)     
        }
        getSteps();
        

    },[dispatch])

    if(!stops.isDownloaded){
        return (
            <main>
            <BreadCrumbs />
            <Preloader />
            </main>
            )
    }
    else
    return (
        <>
        <main>
            <BreadCrumbs />
            <section className="mb0">                        
                <div className="row mb0">
                    <div className="input-field col s12 mb0">
                        <h5 ref={titulo}>Paradas</h5>
                    </div>
                </div>
            </section>
            <div className="mt10 pAll10 ">
                <div className="row">
                    <section className="col s12 m12 pAll0 ">
                        <div className="white black-text pAll0 animate__animated animate__fadeIn">
                        {
                            (stops.isDownloaded) && <MapLines paradas={stops.data} focus={foco} walkingPath={jsonWalkingPath}/>
                        }
                        </div>
                    </section>                
                </div>
                <div className="mt10 ">
                    <div className="row">                                            
                        <section className="col s12 m12 l12 xl6">
                            <div className="card altoMinimo animate__animated animate__slideInUp">
                                <div className="card-content p0">
                                    <header className="card-title white-text line1 center-align card-title-text">
                                        <p className='tituloResultados' >Paradas desde {
                                        //Origen
                                        stops.data[0]?.name
                                        } hasta {
                                        // Destino
                                        stops.data[stops.data?.length-1]?.name

                                        } </p> <i className="small material-icons">timeline</i>
                                    </header>
                                    <main className="white black-text pl20 pr20 pt20 pb20">
                                        <input placeholder="Filtro de paradas" id="stop_name" type="text" className="validate"
                                        name="stopName" value={ stopName } onChange={ handleSearchInputChange }
                                        />
                                        <label htmlFor="stop_name">Nombre de parada</label>          
                                        {stopsFiltered?.length > 0 && <StopsLine search={stopsFiltered} handleClick={handleClick} />}
                                    </main>
                                </div>
                            </div>
                        </section>                    
                        <section className="col s12 m12 l12 xl6">
                            <div className="card altoMinimo animate__animated animate__slideInUp">
                                <div className="card-content p0">
                                    <header className="card-title white-text line1 center-align card-title-text">
                                        <p className='tituloResultados'>Ruta a pie hasta {
                                            // Origen
                                            foco?.name || stops.data[0]?.name
                                        } </p> <i className="small material-icons">directions_walk</i>
                                    </header>
                                    <main className="white black-text pl20 pr20 pt20 pb20">
                                        { 
                                        
                                        jsonWalkingPath && <WalkPath 
                                                                    toOriginSteps={jsonWalkingPath.Origen.routes[0].legs[0].steps} 
                                                                    durationStepsToOrigin={Math.floor(jsonWalkingPath?.Origen?.routes[0].duration / 60)} 
                                                                    toDestinationSteps={jsonWalkingPath.Destino.routes[0].legs[0].steps} 
                                                                    durationStepsToDestination={Math.floor(jsonWalkingPath?.Destino?.routes[0].duration / 60)} 
                                                            />
                                        
                                        }
                                    </main>
                                </div>
                            </div>
                        </section>
                    </div>
                </div>             
            </div>
        </main>
        </>

    )
}
