import React, { useCallback, useEffect, useRef, useState } from 'react';
import { BreadCrumbs } from '../ui/BreadCrumbs';
import { useDispatch, useSelector } from 'react-redux';
import { lineNotLoaded, oneLineLoading } from '../../actions/lines';
import { MapLines } from '../ui/MapLines';
import { useParams } from 'react-router';
import { StopsLine } from '../ui/StopsLine';
import { getCurrentPosition } from '../../helpers/currentPosition';
import { walkingPathOriginStartLoading, walkingPathResetOrigin } from '../../actions/walkingPath';
import { WalkPath } from '../ui/WalkPath';
import Preloader from '../ui/Preloader';
import { Notifications } from '../ui/Notifications';
import { AddLineToFav } from '../ui/AddLineToFav';
import { startChecking } from '../../actions/auth';
import { busesStartLoading } from '../../actions/buses';


export const ViewLineScreen = () => {
    
    const { codeLine } = useParams();
    const lineStored = useSelector(state => state.line);
    const jsonWalkingPath = useSelector(state => state.walkingPath);
    const {isDownloaded: busesIsDownloaded, data: buses} = useSelector(state => state.buses);

    const dispatch = useDispatch();
    const [paradasOrdenadas, setParadasOrdenadas] = useState([
    {
        longitude: 0,
        latitude: 0
    }])
    const [foco, setFoco] = useState({
        lng: '',
        lat: '',
        name: '-selecciona una parada-'
    })
    const titulo = useRef()
    
    const {uid} = useSelector(state => state.auth)
    useEffect(() => {   
        dispatch(startChecking(uid))
    }, [dispatch, uid])

    useEffect(() => {
        if(!busesIsDownloaded){
            dispatch(busesStartLoading());
        }
    }, [busesIsDownloaded, dispatch]);


    useEffect(() => {
        if(lineStored.codeLine !== codeLine){
            if(lineStored.isDownloaded){
                dispatch(lineNotLoaded())    
            }
            else{
                dispatch(oneLineLoading(codeLine))
            }
            !jsonWalkingPath.ResetedOrigen && dispatch(walkingPathResetOrigin());
        }
    }, [dispatch, codeLine, lineStored, jsonWalkingPath])

    
    // Las paradas se ordenan de principio a fin
    useEffect(() => {
        if(lineStored.isDownloaded && lineStored.codeLine === codeLine){
            if (lineStored?.firstStop && lineStored?.lastStop){
                let firstStop = lineStored?.firstStop;
                let lastStop = lineStored?.lastStop;                
                let listStops = [];   

                const insertStop = stop =>{
                    listStops.push(stop)
                    if(stop?.codeStop !== lastStop.codeStop){
                        let detalles = stop?.details.find(detalle => detalle.line.codeLine === codeLine && detalle?.nextStop)
                        let nextObjStop = lineStored?.stops?.find(stop => stop.codeStop === detalles.nextStop)
                        insertStop(nextObjStop)
                    }
                    return listStops
                }                
                firstStop && setParadasOrdenadas(insertStop(firstStop, listStops))
            }
        }
    }, [lineStored, codeLine])

    
    const handleClick = useCallback((lngLat) => {
        const endCoords = [lngLat.lng, lngLat.lat];            
        const getSteps = async() => {
            const currentPosition = await getCurrentPosition()            
            const startCoords = [currentPosition.coords.longitude, currentPosition.coords.latitude]
            dispatch(walkingPathOriginStartLoading(startCoords, endCoords));
            titulo.current.scrollIntoView({ behavior: "smooth", block: "start"});
            setFoco(lngLat)       
        }
        getSteps();
    },[dispatch])

        
    if(!lineStored.isDownloaded || paradasOrdenadas <=1){
        return (
            <main>
            <BreadCrumbs />
            <Preloader />
            </main>)
    }
    
    else if(lineStored.isDownloaded){
        return (
        <>
        <main>        
            <BreadCrumbs />
            <div className="mb0">                        
                <div className="row mb0">
                    <div className="input-field col s12 mb0 tituloPantalla">
                        <h5 ref={titulo} id='lineNumber'>Línea {codeLine}</h5><AddLineToFav codeLine={codeLine} />                        
                    </div>
                </div>
            </div>
            {
            (busesIsDownloaded && !buses.find(bus => bus?.codeLine === codeLine && bus.online)) &&
            <div className="mb0">                        
                <div className="row mb0 mt0">
                    <div className="input-field col s12 mb0 mt0 tituloPantalla">                    
                        <p className='GPSinfo'><i className='material-icons right'>gps_off</i>No hay autobuses de línea {codeLine} emitiendo su posición</p>                     
                    </div>
                </div>
            </div>
            }
            <div className="mt10 pAll10">
                <div className="row">
                    <section className="col s12 m12 pAll0">
                        <div className="white black-text pAll0 animate__animated animate__fadeIn">
                        {
                            lineStored.isDownloaded && 
                            <MapLines 
                            lineas={[
                                {
                                    path: lineStored?.route?.path, 
                                    color: lineStored?.colorStart, 
                                    desc: lineStored?.codeLine + " - " + lineStored?.from + " - " + lineStored?.to + " - " + lineStored?.from
                                },                                
                            ]} 
                            {...(paradasOrdenadas.length > 0 ? {paradas : paradasOrdenadas} : {})}
                            focus={foco} 
                            walkingPath={jsonWalkingPath}
                            />
                        }
                        
                        </div>
                    </section>                
                </div>
                <div className="mt10 contenedorTitulos">
                    <div className="row">                                            
                        <section className="col s12 m12 l12 xl6">
                            <div className="card animate__animated animate__slideInUp">
                                <div className="card-content p0">
                                    <header className="card-title white-text line1 center-align card-title-text">
                                        <p className="tituloResultados">Itinerario desde {
                                        paradasOrdenadas[0]?.name
                                        } hasta {
                                        paradasOrdenadas[paradasOrdenadas?.length-1]?.name

                                        } </p> <i className="small material-icons">timeline</i>
                                    </header>
                                    <main className="white black-text pl20 pr20 pt20 pb20">           
                                        <StopsLine search={paradasOrdenadas} handleClick={handleClick} />                
                                    </main>
                                </div>
                            </div>
                        </section>                    
                        <section className="col s12 m12 l12 xl6">
                            <div className="card animate__animated animate__slideInUp">
                                <div className="card-content p0">
                                    <header className="card-title white-text line1 center-align card-title-text">
                                        <p className="tituloResultados">Ruta a pie hasta {
                                            // Origen
                                            foco?.name
                                        } </p> <i className="small material-icons">directions_walk</i>
                                    </header>
                                    <main className="white black-text pl20 pr20 pt20 pb20">
                                        { 
                                        
                                        jsonWalkingPath && <WalkPath 
                                                                    toOriginSteps={jsonWalkingPath.Origen.routes[0].legs[0].steps} 
                                                                    durationStepsToOrigin={Math.floor(jsonWalkingPath?.Origen?.routes[0].duration / 60)} 
                                                                    toDestinationSteps={jsonWalkingPath.Destino.routes[0].legs[0].steps} 
                                                                    durationStepsToDestination={Math.floor(jsonWalkingPath?.Destino?.routes[0].duration / 60)} 
                                                            />
                                        
                                        }
                                    </main>
                                </div>
                            </div>
                            <section className="card">
                                <div className="card-content p0">
                                    <header className="card-title white-text line1 center-align card-title-text animate__animated animate__slideInUp">
                                        <p className="tituloResultados">Notificaciones de línea {
                                            codeLine
                                        } </p> <i className="small material-icons">notifications</i>
                                    </header>
                                    <main className="white black-text pl20 pr20 pt20 pb20">
                                        <Notifications lines={[codeLine]} />
                                    </main>                                        
                                </div>
                            </section>                            
                        </section>
                    </div>
                </div>                            
            </div>
        </main>
        </>
    )}  
}
