import React, {memo, useState, useContext, useEffect, useRef} from 'react'
import { SocketContext } from '../../context/SocketContext';
import { useMapbox } from '../../hooks/useMapbox';
import { BreadCrumbs } from './BreadCrumbs';
import { MapFilter } from './MapFilter';
import Preloader from './Preloader';

const initialValues = {
    lng : -6.3633,
    lat : 36.7719,
    zoom : 13.45
}

export const MapLines = memo(({lineas, paradas, focus, walkingPath}) => {
    const {
        setRef,
        //coords,
        newBusMarker$,
        addStop,
        addBusRoute,
        deleteAllBusStops,
        routesRepo$,
        renderiza,
        mapa$,
        changeMapFocus,
        addBusMarker,
        moveBusMarker,
        removeBusMarker
    } = useMapbox(initialValues);

    const [loading, setLoading] = useState(false)

    useEffect(() => {        
        focus?.lng && focus?.lat && 
        setTimeout(() => changeMapFocus(focus.lng, focus.lat), 1000);
    }, [focus, changeMapFocus])
    
    // Incluimos paradas
    // Las paradas pueden llegar de search o de una
    // o varias líneas en concreto
    useEffect(() => { 
        deleteAllBusStops();
        if(paradas?.length > 0){
            setLoading(true)           

            for (let stop of paradas) {
                stop?.longitude && stop?.latitude && addStop(stop)
            }

            setLoading(false)           

        }
    }, [paradas, addStop, deleteAllBusStops])

    /* AÑADE LAS RUTAS DE LAS LÍNEAS */    
    const preLines = useRef([]);
    useEffect(() => {
        const equals = (a, b) => JSON.stringify(a) === JSON.stringify(b);
        if(lineas?.length && !equals(lineas, preLines.current)){
            setLoading(true)           
            for(let linea of lineas){
                addBusRoute(linea.path, linea.color, linea.desc)                
            }
            preLines.current = lineas;
            setLoading(false)
        } else {
            //addBusRoute('','','')
        }
    }, [lineas, addBusRoute])

    /* AÑADE RUTA A PIE A PARADA SEÑALADA*/
    const preRouteOri = useRef([])
    useEffect(() => {
            const equals = (a, b) => JSON.stringify(a) === JSON.stringify(b);
            if(walkingPath?.Origen?.routes[0]?.geometry?.coordinates?.length > 0){                
                if(!equals(walkingPath?.Origen?.routes[0]?.geometry?.coordinates, preRouteOri.current)){
                    addBusRoute(walkingPath?.Origen.routes[0].geometry.coordinates, '#0f8', 'Ruta a pie a parada')                    
                    preRouteOri.current = walkingPath?.Origen.routes[0].geometry.coordinates;
                }
            }
            else{
                addBusRoute([], '#0f8', 'Ruta a pie a parada')
            }
    }, [addBusRoute, walkingPath?.Origen?.routes])

    
    // Socket para escuchar nuevos buses
    const {socket} = useContext(SocketContext) 

    // Observador de nuevo marcador
    useEffect(() => {        
        newBusMarker$.subscribe(marker => {
            socket.emit('marcador-nuevo', marker) 
            return ()=> socket.off("marcador-nuevo");           
        })
    }, [newBusMarker$, socket])
    
    // Escuchar nuevos marcadores que vienen del server
    useEffect(() => {
        socket.on('marcador-nuevo', (marcador)=>{
            lineas?.find(linea => linea?.desc?.replace(/\D/g,'') === marcador?.line?.toString()) && addBusMarker(marcador, marcador.id, marcador?.line)
        })
        return ()=> socket.off("marcador-nuevo");           

    }, [socket, lineas, addBusMarker])

    // Observador de nuevo movimiento
    /*useEffect(() => {
        markerMovement$.subscribe(marker => {
            socket.emit('marcador-actualizado', marker)
        })
    }, [markerMovement$, socket])*/

    useEffect(() => {
        socket.on('marcador-actualizado', (marcador)=>{
            marcador?.lng && marcador?.lat && lineas?.find(linea => linea?.desc?.replace(/\D/g,'') === marcador?.line?.toString()) && moveBusMarker(marcador)
        })
        return ()=> socket.off("marcador-actualizado");           

    }, [socket, lineas, moveBusMarker])

    useEffect(() => {
        socket.on('marcador-borrado', (identification)=>{
            removeBusMarker(identification)
        })
        return ()=> socket.off("marcador-borrado");           

    }, [socket, lineas, removeBusMarker])


    const handleSwitch = e => {
        const {value, checked} = e.target;
        
        if(!e.target.checked){
            e.checked = false;
        } 
        !checked && mapa$.setLayoutProperty(value, 'visibility', 'none');
        checked && mapa$.setLayoutProperty(value, 'visibility', 'visible');
    }

    // Marcadores existentes
    const marcadoresActivos = useRef([])
    // Mostrar los activos
    useEffect(() => {
        if(!marcadoresActivos.current.length){
            socket.emit('mostrar-activos')

        }
        return ()=> socket.off("marcadores-activos");           

    }, [socket])

    useEffect(() => {
        socket.once('marcadores-activos', marcadores => {
            for( const key of Object.keys( marcadores ) ) {
                lineas?.find(linea => linea?.desc?.replace(/\D/g,'') === marcadores[key]?.line?.toString()) && addBusMarker( marcadores[key], key, marcadores[key]?.line );
            }
            marcadoresActivos.current = marcadores;
        })
        return ()=> socket.off("marcadores-activos");           

    }, [socket, lineas, addBusMarker])

    if(loading){
        return (
            <main>
                <BreadCrumbs />
                <Preloader />
                <div>
                <h1>Cargando</h1>
                </div>
            </main>
        )
    }else
    return (
        
        <div id="mapComponent" className="mapComponent">
            <div
                ref={setRef}
                className="mapContainer"
            />
            <div className="infoWindow">
                {
                //lng: {coords.lng} | lat: {coords.lat} | zoom: {coords.zoom}
                }
            </div>
            {
            routesRepo$.length > 0 &&
            <div className="mapLayer">
                <ul>
                    {
                    <MapFilter routesRepository={renderiza} mapa={mapa$} handleFunction={handleSwitch} />
                    }
                </ul>
            </div>
            }


        </div>
        
    )
})