import React, {useEffect, useRef} from 'react'
import { useMapbox } from '../../hooks/useMapbox';
import { useSelector } from 'react-redux';
import { MapFilter } from './MapFilter';

const initialValues = {
    lng : -6.3633,
    lat : 36.7719,
    zoom : 13.45
}

export const MapUI = ({focus}) => {
    const paradas = useSelector(state => state.search);      
    const walkingPath = useSelector(state => state.walkingPath);
    const {
        setRef,
        //coords,
        addStop,
        addBusRoute,
        deleteAllBusStops,
        mapa$,       
        renderiza,
        changeMapFocus
        } = useMapbox(initialValues);
    
    useEffect(() => {        
        focus?.lng && focus?.lat && changeMapFocus(focus.lng, focus.lat)
    }, [focus, changeMapFocus])
    
    // Incluimos paradas
    // Las paradas pueden llegar de search o de una
    // o varias líneas en concreto
    useEffect(() => { 
        deleteAllBusStops();
        if(paradas.length > 0){
            for (let parada of paradas) {
                addStop(parada)
            }
        }
    }, [paradas, addStop, deleteAllBusStops])

    // preparamos useRef para comparar con estados anteriores
    const preRouteOri = useRef([]);
    const preRouteDest = useRef([]);
    const preParadas = useRef([]);

    const equals = (a, b) => JSON.stringify(a) === JSON.stringify(b);

    useEffect(() => {
        //Borramos lo que había
        // Solo añadimos si el estado anterior es diferente del de ahora
        
        if( paradas.length > 0){
            //El color hay que cambiarlo en función de si es una línea lo que mostramos o es un searchResults        

            if(!equals(paradas.map(e => [e.longitude, e.latitude]), preParadas.current)){
                addBusRoute(paradas.map(e => [e.longitude, e.latitude]), '#f0f', 'Ruta encontrada')
                preParadas.current = paradas.map(e => [e.longitude, e.latitude]);

                setTimeout(() => {
                    mapa$?.flyTo({                
                        zoom: 13
                    });
                    setTimeout(() => {
                        mapa$?.flyTo({                           
                            center: [paradas[paradas.length-1].longitude, paradas[paradas.length-1].latitude],
                        });
                        setTimeout(() => {
                            mapa$?.flyTo({
                                zoom: 17
                            });
                        }, 1500);  
                    }, 1000);                                    
                 }, 500);
            }
        }
        
    }, [addBusRoute, walkingPath.Destino.routes, walkingPath.Origen.routes, paradas, mapa$])

    // La ruta de origen está determinada por el código de parada 
    // aunque se escriba a mano un origen que no sea parada.
    // porque al final se obtiene la ruta a una parada más cercana.
    useEffect(() => {
        if( paradas.length ){
            if(walkingPath?.Origen?.routes[0]?.geometry?.coordinates?.length > 0){
                
                if(!equals(walkingPath?.Origen?.routes[0]?.geometry?.coordinates, preRouteOri.current)){
                    addBusRoute(walkingPath.Origen.routes[0].geometry.coordinates, '#0f8', 'Ruta a pie a parada')
                    preRouteOri.current = walkingPath.Origen.routes[0].geometry.coordinates;
                }
            }
            else{
                addBusRoute([], '#0f8', 'Ruta a pie a parada')
            }
        }

    },[addBusRoute, paradas, walkingPath.Origen.routes])

    // La ruta de destino está determinada por lo que se escriba en el campo de búsqueda.
    // Se resetea en cada búsqueda.
    useEffect(() => {
        if(walkingPath?.Destino?.routes[0]?.geometry?.coordinates?.length > 0 ){
            addBusRoute(walkingPath.Destino.routes[0].geometry.coordinates, '#52f', 'Ruta a pie a destino')
            preRouteDest.current = walkingPath.Destino.routes[0].geometry.coordinates;
            
            
        }
        else{
            //reset de rutas a destino
            addBusRoute([], '#52f', 'Ruta a pie a destino')
        }
    },[addBusRoute, walkingPath.Destino.routes])

    const handleSwitch = e => {
        const {value, checked} = e.target;
        !checked && mapa$.setLayoutProperty(value, 'visibility', 'none');
        checked && mapa$.setLayoutProperty(value, 'visibility', 'visible');
    }

    return (
        <div className="mapComponent">
            <div
                ref={setRef}
                className="mapContainer"
            />
            <div className="infoWindow">
                {
                //lng: {coords.lng} | lat: {coords.lat} | zoom: {coords.zoom}
                }
            </div>
            <div className="mapLayer">
                <ul> 
                    {                  
                    <MapFilter routesRepository={renderiza} mapa={mapa$} handleFunction={handleSwitch} />                    
                    }
                </ul>
            </div>
        </div>
    )
}