import React, {useEffect, useRef} from 'react'
import { useLocation } from 'react-router-dom';

import M from "materialize-css/dist/js/materialize.min.js";
import "materialize-css/dist/css/materialize.min.css";
import { useSelector, useDispatch } from 'react-redux';
import { startLogout } from '../../actions/auth';
import { Link } from 'react-router-dom';
import 'animate.css';


const Navbar = () => {
    const location = useLocation();
    const sidenav = useRef()


    useEffect(()=>{

        const instancia = M.Sidenav.getInstance(sidenav.current);
        let dragTargetElement = document.querySelector(".drag-target");
        if(dragTargetElement){            
            dragTargetElement.remove();
        }

        if(!instancia){
            M.Sidenav.init(sidenav.current, {
                edge : 'left',
                preventScrolling: true,
                draggable: true
            });
        }
    },[])
    useEffect(() => {
        var elems = document.querySelectorAll('.collapsible');
        M.Collapsible.init(elems);
    },[])
    
    const {name, rol} = useSelector(state => state.auth);

    const dispatch =  useDispatch();
        
    const handleLogout = () => {
        dispatch(startLogout());
    }
    
   return (   
    <div>

        <header>
        <nav>
            <div className="nav-wrapper navColor">
                <div className="row">
                    <div className="col s12">
                    <a href="#!" data-target="sidenav-1" className="left sidenav-trigger hide-on-medium-and-up"><i className="material-icons">menu</i></a>
                    <ul id="nav-mobile" className="centerElements right  hide-on-med-and-down">
                        <li><Link to="/">  <i className="material-icons right right">search</i></Link></li>

                    </ul>
                    </div>
                </div>
            </div>
        </nav>
        </header>
        <ul ref={sidenav} id="sidenav-1" className="sidenav sidenav-fixed">
            <li>
                <Link to="/profile">
                <figure className="circle">
                    <img className="circle" src={name === 'Cristian' ? '/images/foto_perfil_150.jpg' : '/images/default_avatar.png'}
                    alt="" />                            
                </figure>
                </Link>
                        
                <Link to="/profile"><span className="white-text name">{name}</span></Link>
            </li>
            <li>
                <figure className="logo">
                    <Link to="/main"><img src="/images/logo.svg" alt="Logo de Bahía Bus" className="responsive-img" /></Link>
                </figure>
                
            </li>
            <li><div className="divider" /></li>
            <li className={( location.pathname.includes("/profile")) ? 'active' : ''}><Link to="/profile"><i className="material-icons right">person</i>Mi Cuenta</Link></li>
            <li className={( location.pathname.includes("/notifications")) ? 'active' : ''}><Link to="/notifications"><i className="material-icons right">notifications</i>Notificaciones</Link></li>
            <li><div className="divider" /></li>
            <li><a className="subheader" href="#!">Opciones</a></li>            
            <li className={( location.pathname.includes("/lines") && !location.pathname.includes("/admin")) ? 'active' : ''}><Link className="waves-effect" to="/lines">Líneas<i className="material-icons right">directions_bus</i></Link></li>
            <li className={( location.pathname.includes("/stops") && !location.pathname.includes("/admin")) ? 'active' : ''}><Link className="waves-effect" to="/stops">Paradas<i className="material-icons right">store</i></Link></li>
            <li className={( location.pathname.includes("/favorites")) ? 'active' : ''}><Link className="waves-effect" to="/favorites">Favoritos<i className="material-icons right">favorite</i></Link></li> 
            {
                rol === "ADMIN_ROLE" &&
            
            <li><ul className="collapsible collapsible-accordion">
                    <li className={( location.pathname.includes("/admin")) ? 'active' : ''} >
                    <a href="#!" onClick={e=>e.preventDefault()} className="waves-effect collapsible-header">Administración
                        <i className="material-icons right">arrow_drop_down</i>
                    </a>
                    <div className="collapsible-body">
                        <ul>
                            <li className={( location.pathname.includes("/admin/stops")) ? 'active' : ''}><Link to="/admin/stops">Paradas<i className="material-icons right red-text lighten-2">store</i></Link></li>
                            <li className={( location.pathname.includes("/admin/lines")) ? 'active' : ''}><Link to="/admin/lines">Líneas<i className="material-icons right red-text lighten-2">linear_scale</i></Link></li>
                            <li className={( location.pathname.includes("/admin/buses")) ? 'active' : ''}><Link to="/admin/buses">Autobuses<i className="material-icons right red-text lighten-2">directions_bus</i></Link></li>
                            <li className={( location.pathname.includes("/admin/routes")) ? 'active' : ''}><Link to="/admin/routes">Rutas<i className="material-icons right red-text lighten-2">timeline</i></Link></li>
                            <li className={( location.pathname.includes("/admin/notifications")) ? 'active' : ''}><Link to="/admin/notifications">Notificaciones<i className="material-icons right red-text lighten-2">notifications</i></Link></li>
                            <li className={( location.pathname.includes("/admin/users")) ? 'active' : ''}><Link to="/admin/users">Usuarios<i className="material-icons right red-text lighten-2">person</i></Link></li>
                        </ul>
                    </div>
                    </li>
                </ul>           
            </li>
            }
            <li><div className="divider" /></li>
            {
                rol === "BUS_ROLE" &&
                <li><Link to="/bus"><i className="material-icons right">pin_drop</i>Demo Buses</Link></li>
            }
            <li><Link to="#!" onClick={handleLogout}><i className="material-icons right">exit_to_app</i>Salir</Link></li>
        </ul>      
    </div>   
   )
}


export default Navbar
