import React, {useEffect} from 'react';
import { useHistory, useLocation } from 'react-router-dom';
import M from "materialize-css/dist/js/materialize.min.js";
import { useDispatch, useSelector } from 'react-redux';
import { stopsStartLoading } from '../../actions/stops';
import { useParams } from "react-router-dom";
import { stopsSearchReset, stopsSearchStartLoading } from '../../actions/search';
import { nearestDestinationStopStartLoading, nearestOriginStopStartLoading, nearestStopsReset } from '../../actions/nearestStop';
import Swal from 'sweetalert2';


export const SearchPathForm = () => { 
    
    const dispatch = useDispatch();
    const stops = useSelector(state => state.stops);    
    const location = useLocation();

    useEffect(() => {
        if(!location.pathname.includes("searchresults")){
            dispatch(stopsSearchReset())
        }
    }, [dispatch, location])

    useEffect(() => { 
        !stops.isDownloaded && dispatch(stopsStartLoading())
    }, [dispatch, stops]);

    const { place1, place2 } = useParams();

    useEffect(() => {
        if(place1 && place2)      
            dispatch(stopsSearchStartLoading(place1, place2));        
    }, [dispatch, place1, place2])

    useEffect(() => {
        if(stops.isDownloaded){
            let elems = document.querySelectorAll('.autocomplete');
            const allStops = stops.data.map(stop =>[stop.name, null]);
            const datos = Object.fromEntries(allStops);
            let options = {
                data: datos,
                limit: 5
            }
            
            M.Autocomplete.init(elems, options);                    
        }
    }, [stops])

    const history = useHistory();

    const handleSearch = (e) =>{
        e.preventDefault();
        
            if(!e.target.Origen.value || !e.target.Destino.value ){
                return
            }
            if(stops.isDownloaded){
                let Origen = stops.data.find(stop => stop.name === e.target.Origen.value )?.codeStop;
                let Destino = stops.data.find(stop => stop.name === e.target.Destino.value )?.codeStop;

                /*
                Si no hay origen o destino, buscarlo de forma asíncrona
                Si lo hay, hacerlo como siempre
                */
                dispatch(nearestStopsReset());
                if(!Origen || !Destino){
                    const getOrigenYDestino = async() => {
                        try {
                            if(!Origen){
                                const Ori = await nearestOriginStopStartLoading(e.target.Origen.value, dispatch);
                                if(Ori){
                                    Origen = Ori.CodParada;
                                }
                                else{
                                    Swal.fire({
                                        icon: 'error',
                                        title: 'Oops...',
                                        text: 'No se pudo encontrar el origen',
                                        footer: 'Inténtalo de nuevo con otro nombre'
                                    })
                                    return;
                                }
                            }
                            if(!Destino){
                                const Dest = await nearestDestinationStopStartLoading(e.target.Destino.value, dispatch);
                                if(Dest){
                                    Destino = Dest.CodParada;
                                }
                                else {
                                    Swal.fire({
                                        icon: 'error',
                                        title: 'Oops...',
                                        text: 'No se pudo encontrar el destino',
                                        footer: 'Inténtalo de nuevo con otro nombre'
                                    });
                                    return
                                }                            
                            }

                            if(!location.pathname.includes(`/searchresults`)){          
                                history.push(`/searchresults/${Origen}/${Destino}`);
                            }else{                            
                                dispatch(stopsSearchStartLoading(Origen, Destino));
                                window.history.replaceState(null, `Resultados de ${Origen} hasta ${Destino}`, `/searchresults/${Origen}/${Destino}`)                        
                            }
                        }
                        catch(error){
                            Swal.fire({
                                icon: 'error',
                                title: 'Oops...',
                                html: error,
                                footer: 'Ha ocurrido un error'
                            })
                        }
                        
                    }                
                    getOrigenYDestino()
                }
                else{
                    
                    if(!location.pathname.includes(`/searchresults`)){          
                        history.push(`/searchresults/${Origen}/${Destino}`);
                    }else{
                        dispatch(stopsSearchStartLoading(Origen, Destino));
                        window.history.replaceState(null, `Resultados de ${Origen} hasta ${Destino}`, `/searchresults/${Origen}/${Destino}`)                        
                    }
                }
            }
    }

     return (
        <div className="mt0 pt50 pb50 backgroundSearch mb0 white white-text text-lighten-1 backgroundSearchShadow animate__animated animate__fadeIn">
        <div className="pt130 pb130 pr0 pl0 relative">                    
            <div className="container pAll15 searchForm">
            <div className="row mb0">
                <h2 className="white-text amber-text text-lighten-5 center-align">¿Adónde quieres ir?</h2>
            </div>
                <div className="row mb0">
                    <form className="col s12" onSubmit={ handleSearch }>
                        <div className="row mb0 flexForm">
                            <div className="input-field col mr0 pAll0 input100">
                                <input placeholder="Origen" type="text" id="origen" className="autocomplete grey-text text-lighten-1 originInput pl10" name="Origen" />

                            </div>
                            <div className="input-field col  pl0 pr0 input100">
                                <input placeholder="Destino" type="text" id="destino" className="autocomplete grey-text text-lighten-1 destinationInput pl10" name="Destino"  />

                            </div>                            
                            <div className="input-field col left-align pAll0">
                                <button className="btn waves-effect waves-light orange lighten-2  waves-effect waves-light searchButton" type="submit" name="action">
                                    <i className="material-icons center">search</i>
                                </button>                            
                            </div>
                        </div>
                    </form>
                </div>
            </div>                             
        </div>
    </div>
    )
}
