import React, { useEffect, useRef } from 'react'
import { useForm } from '../../../hooks/useForm';
import M from "materialize-css/dist/js/materialize.min.js";
import {useDispatch, useSelector } from 'react-redux';
import { closeModal } from '../../../actions/modal';
import { startCloseFormMode, startDataReadyFormMode } from '../../../actions/form';
import { busSavingNew, startEditingBus } from '../../../actions/buses';

export const CrudBusForm = ({allLines}) => {
    const modal = useSelector(state => state.modal)
    const form = useSelector(state => state.form)

    // Inicialización de formulario
    const [ formLineValues, handleLineInputChange, reset, setValuesManually ] = useForm({
        identification : '',
        line : ''  
    });

    const {
        identification,
        line        
    } = formLineValues;


    const dispatch = useDispatch()   
    const selectLines = useRef();

    // Cargamos materializecss
    useEffect(() => {
        M.FormSelect.init(selectLines.current, {});
    })
    
    // Envío de datos al server
    const handleLineSubmit = (e) => {
        e.preventDefault();        

        // Preparamos lo que vamos a enviar
        let codeLine = '';
        if(line?.codeLine)
        {
            codeLine = line?.codeLine;
        }
        else{
            const linex = allLines.find(l => l?._id.toString() === line.toString())
            codeLine = linex?.codeLine;
        }
        const datosAMandar = {...formLineValues, codeLine: codeLine}

        // Despachamos en función de lo que hagamos (Editar | Nuevo)
        if(form.edit){
            dispatch(startEditingBus(datosAMandar))
        }
        else if(form.new) {
            dispatch(busSavingNew(datosAMandar))
        }        
        // RESET antes de irse
        resetForm()
    }

    const resetForm = (e)=>{
        e?.preventDefault();
        reset()
        // Reseteo el selector de rutas
        selectLines.current.selectedIndex = -1;        
        dispatch(closeModal());
        dispatch(startCloseFormMode())
    }    
        
    /* Modo Edición y cargamos valores */
    useEffect(() => {
        // Aquí cargo datos si el formulario aún no está ready 
        // (si estamos en modo edición y además hay actividades)
        if( modal.data && form.edit && !form.dataReady){
            // Convierto a objeto el array de period of time para su tratamiento
            // Ya que es mejor utilizar campos únicos que claves numéricas procedentes
            // del index de cada posición del array
            setValuesManually({...modal.data})
        }   
    }, [modal.data, form, dispatch, setValuesManually])

    /* Si está en modo edición, cargamos los selects*/
    useEffect(() => {
        if(form.edit && !form.dataReady){
            // Dejamos a ready el formulario una vez que ya está cargado
            dispatch(startDataReadyFormMode())
        }
    }, [dispatch, form, modal])

    return (
        <div id="LinesForm" className="col s12">
            <form className="col s12" onSubmit={ handleLineSubmit }>
            
            <div className="form-container">
                <h3 className="teal-text">Autobús</h3>
                <div className="row">
                    <div className="input-field col s12 ">
                        <i className="material-icons prefix">description</i>
                        <input id="identification" autoComplete="identification" type="text" className="validate" name="identification" value={identification} onChange={ handleLineInputChange } />

                        <label className="active" htmlFor="identification">Identificación</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">directions_bus</i>
                        <select ref={selectLines} value={line._id} onChange={ handleLineInputChange } name="line" >
                            <option value='' defaultValue=''>Elige una opción para línea</option>
                            {
                                allLines.map(line => (<option key={line._id} value={line._id}>{line.codeLine} </option> ))
                            }                            
                        </select>
                        <label className="active" htmlFor="from">Línea a la que pertenece</label>
                    </div>
                </div>              
                <br />
                <div className="center-align animate__fadeInDown">
                    <button className="btn waves-effect waves-light teal modal-close" type="submit" name="action">
                        <i className="material-icons white-text text-darken-4 left ">save</i>Guardar
                    </button>
                    <button to="#!" className="ml15 modal-close red darken-red-3 waves-effect waves-red btn" onClick={resetForm}>
                        <i className="material-icons left">cancel</i>Cancelar
                    </button>
                </div>
            </div>
            </form>
        </div>
    )
}
