import React, { useEffect, useRef } from 'react'
import { useForm } from '../../../hooks/useForm';
import M from "materialize-css/dist/js/materialize.min.js";
import {useDispatch, useSelector } from 'react-redux';
import { closeModal } from '../../../actions/modal';
import { startCloseFormMode } from '../../../actions/form';
import { notificationSavingNew } from '../../../actions/notification';
import { Link } from 'react-router-dom';
import Swal from 'sweetalert2';

export const CrudBusesNotificationsForm = ({allBuses}) => {
    const form = useSelector(state => state.form)

    // Inicialización de formulario
    const [ formBusNotificationValues, handleBusNotificationInputChange, reset ] = useForm({
        description : '',
        bus : '',
    });

    const {
        description,
        bus,
    } = formBusNotificationValues;


    const dispatch = useDispatch()    
    const selectBuses = useRef();
    const pathRef = useRef();    

    // Cargamos materializecss
    useEffect(() => {
        M.FormSelect.init(selectBuses.current, {});
    })
    
    // Envío de datos al server
    const handleBusNotificationSubmit = (e) => {
        e.preventDefault();        
        // Recorre todos los periodos de actividad y los transforma en arrays normales
        // Se preparan los stops seleccionados
        const auxResultado = allBuses?.find(busy => busy._id === bus);
        if(auxResultado){
            const {identification, line} = auxResultado;
            const {codeLine} = line;
            // Preparamos lo que vamos a enviar
            const datosAMandar = {description: "<" + identification + "> " + description, notificationType: 'Bus', busServiceId: bus, codeLine }

            // Despachamos en función de lo que hagamos (Editar | Nuevo)            
            if(form.new) {
                dispatch(notificationSavingNew(datosAMandar))
            }        
            // RESET antes de irse
            resetForm()
        }
        else{
            Swal.fire('Faltan datos', 'Rellena todos los campos', 'warning');
        }
    }

    const resetForm = (e)=>{
        e?.preventDefault()        
        reset(); // para el        
        // Reseteo el selector de rutas
        selectBuses.current.selectedIndex = -1;        
        dispatch(closeModal());
        dispatch(startCloseFormMode())        
    }
    
    const changedValue = () => {
        pathRef.current.style.overflowY = "scroll";
    }
    
    return (
        <div id="busForm" className="col s12">
            <form className="col s12" onSubmit={ handleBusNotificationSubmit }>
            
            <div className="form-container">
                <h3 className="teal-text">Notificación de autobús</h3>
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">directions_bus</i>
                        <select ref={selectBuses} name="bus" onChange={handleBusNotificationInputChange} value={bus}>
                            <option value="" disabled defaultValue="Elige una opción">Elige una opción</option>
                                {
                                    allBuses?.map(bus => (
                                         <option key={bus?._id} value={bus?._id}>{bus?.identification} (Línea: {bus?.line?.codeLine})</option>
                                    ))
                                }                            
                        </select>
                        <label>Selecciona todas el autobús</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12 ">
                        <i className="material-icons prefix">description</i>
                        <textarea ref={pathRef} id="description" autoComplete="description" rows="10" className="materialize-textarea" name="description" value={description} onChange={ handleBusNotificationInputChange } onClick={changedValue} />
                        <label className="active" htmlFor="description">Descripción.</label>
                    </div>
                </div>    
                <br />
                <div className="center-align animate__fadeInDown">
                    <button className="btn waves-effect waves-light teal modal-close" type="submit" name="action">
                        <i className="material-icons white-text text-darken-4 left ">save</i>Guardar
                    </button>
                    <Link to="#!" className="ml15 modal-close red darken-red-3 waves-effect waves-red btn" onClick={resetForm}>
                        <i className="material-icons left">cancel</i>Cancelar
                    </Link>
                </div>
            </div>
            </form>
        </div>
    )
}
