import React, { useCallback, useEffect, useRef, useState } from 'react'
import { useForm } from '../../../hooks/useForm';
import M from "materialize-css/dist/js/materialize.min.js";
import {useDispatch, useSelector } from 'react-redux';
import { LinesDates } from './LinesDates';
import {v4} from 'uuid';
import { closeModal } from '../../../actions/modal';
import { lineSavingNew, startEditingNew } from '../../../actions/lines';
import { startCloseFormMode, startDataReadyFormMode } from '../../../actions/form';

export const CrudLinesForm = ({allStops}) => {
    const modal = useSelector(state => state.modal)
    const form = useSelector(state => state.form)

    // Inicialización de formulario
    const [ formLineValues, handleLineInputChange, reset, setValuesManually ] = useForm({
        codeLine : '',
        from : '',
        to : '',
        colorStart : '',
        colorEnd : '',
        periodOfActivity: {},
        firstStop: '',
        lastStop: '',
        buses: []
    });

    const {
        codeLine,
        from, 
        to, 
        colorStart,
        colorEnd,
        firstStop,
        lastStop
        
    } = formLineValues;


    const dispatch = useDispatch()
    const stopsMultipleSelect = useRef([])
    const stopOriginMultipleSelect = useRef(null)
    const stopDestinationMultipleSelect = useRef(null)
    const selectStops = useRef();
    const handleStopsMultipleSelect = (e) => {        
        let arr = [];
        for(let option in e.target.selectedOptions){            
            e.target.selectedOptions[option]?.value !== undefined && (arr = [...arr, e.target.selectedOptions[option]?.value])
        }
        stopsMultipleSelect.current = arr;
    }

    // Cargamos materializecss
    useEffect(() => {
        M.FormSelect.init(selectStops.current, {});
        M.FormSelect.init(stopOriginMultipleSelect.current, {});
        M.FormSelect.init(stopDestinationMultipleSelect.current, {});
    })
    
    // Envío de datos al server
    const handleLineSubmit = (e) => {
        e.preventDefault();                
        // Recorre todos los periodos de actividad y los transforma en arrays normales
        let arrayPeriods = [];
        for(let p in actividades.periodOfActivity){
            arrayPeriods = [...arrayPeriods, actividades.periodOfActivity[p]]
        }
        // Se preparan los stops seleccionados
        const stopsSelected = stopsMultipleSelect.current

        // Preparamos lo que vamos a enviar
        const actividadesAMandar = {...formLineValues, periodOfActivity: arrayPeriods, stops: stopsSelected}

        // Despachamos en función de lo que hagamos (Editar | Nuevo)
        if(form.edit){
            dispatch(startEditingNew(actividadesAMandar))
        }
        else if(form.new) {
            dispatch(lineSavingNew(actividadesAMandar))
        }        
        // RESET antes de irse
        resetForm()
    }

    const resetForm = (e)=>{
        e?.preventDefault()
        reset() // para el
        serviceTimeRef.current = [];
        stopsMultipleSelect.current = []

        // Reseteo el selector de paradas
        selectStops.current.selectedIndex = -1;

        setfechasServicio([]);
        dispatch(closeModal());
        dispatch(startCloseFormMode())
        setActividades({})
    }
    
    // Tomo el valor del select
    const getDatePickerValue = useCallback((sel) => {        
        const {id, nombre, valueSelected} = sel;
        setActividades(prev =>{
            return {   
                ...prev,
                periodOfActivity:{
                    ...prev.periodOfActivity,
                    [id]:{
                        ...((prev?.periodOfActivity?.hasOwnProperty(id)) && prev?.periodOfActivity[id]),
                        [nombre]: valueSelected 
                    }
                }
            }
        })
    },[])

    // Guardamos nuevo periodo
    const [actividades, setActividades] = useState({})
    
    /* Se borra un periodo */
    const delActivity = useCallback((id) => {
        // Animación de borrado
        serviceTimeRef.current[id].className="card animate__animated animate__fadeOutDown";
        setTimeout(()=>{
            // Borramos la instancia del DOM
            setfechasServicio(prev => prev.filter(fechaServicio=> fechaServicio.key !== id?.toString()))
            // Borramos la referencia de dicha instancia
            delete serviceTimeRef.current[id]

            // Se borra del array actividades
            setActividades(prev =>{
                let periodosAux = {};
                for(let period in prev.periodOfActivity){
                    (period.toString() !== id.toString()) && 
                    (
                        periodosAux = {
                            ...periodosAux,
                            [period]: prev?.periodOfActivity[period]
                        }
                    )
                }
                return {   
                    ...prev,
                    periodOfActivity: periodosAux
                }
            })

        }, 1000)       
    },[])

    /* Añadir fecha y tiempo de servicio */
    const [fechasServicio, setfechasServicio] = useState([]);  

    const serviceTimeRef = useRef([]);
    const addActivity = useCallback( (activity, id = null) => {
        // ID único para cada periodo de servicio
        const idFechaServicio = id || v4();
          
          setfechasServicio( prev => [
            ...prev, 
            <LinesDates 
            ref={el => serviceTimeRef.current[idFechaServicio] = el} 
            getDatePickerValue={getDatePickerValue}
            delActivity={delActivity}
            key={idFechaServicio}
            idFechaServicio={idFechaServicio}            
            clase="card animate__animated animate__fadeInDown"
            activity={(activity?.initDate) && activity}
            />
        ])

    }, [delActivity, getDatePickerValue])

    /* Modo Edición y cargamos valores */
    useEffect(() => {
        // Aquí cargo datos si el formulario aún no está ready 
        // (si estamos en modo edición y además hay actividades)
        if( modal.data && form.edit && !form.dataReady){
            // Convierto a objeto el array de period of time para su tratamiento
            // Ya que es mejor utilizar campos únicos que claves numéricas procedentes
            // del index de cada posición del array
            const obj = Object.assign({}, modal.data.periodOfActivity);
            setActividades({periodOfActivity: obj})
            // Cargo el formulario con los actividades del modal
            //setStopsMultipleSelect(modal.data.stops)
            stopsMultipleSelect.current = modal.data.stops;            
            setValuesManually({...modal.data, buses: modal.data.buses, route: modal.data?.route?._id });
        }   
    }, [modal, setValuesManually, form, dispatch])

    /* Si está en modo edición, cargamos los selects*/
    useEffect(() => {
        if(form.edit && !form.dataReady){
            // Vamos añadiendo actividades al DOM.
            // Las actividades tienen tratamiento especial.
            for(let p in modal.data.periodOfActivity){
                addActivity(modal.data.periodOfActivity[p], p)
            }
            // Dejamos a ready el formulario una vez que ya está cargado
            dispatch(startDataReadyFormMode())
        }
    }, [dispatch, form, addActivity, modal])
    
    return (
        <div id="LinesForm" className="col s12">
            <form className="col s12" onSubmit={ handleLineSubmit }>
            
            <div className="form-container">
                <h3 className="teal-text">Línea</h3>
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">directions_bus</i>
                        <input id="codeLine" autoComplete="codeLine" type="text" className="validate" name="codeLine" value={codeLine} onChange={ handleLineInputChange } />
                        <label className="active" htmlFor="codeLine">Código de línea</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">place</i>
                        <input id="from" autoComplete="from" type="text" className="validate" name="from" value={from} onChange={ handleLineInputChange } />
                        <label className="active" htmlFor="from">Origen</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">place</i>
                        <input id="to" autoComplete="to" type="text" className="to" name="to" value={to} onChange={ handleLineInputChange } />
                        <label className="active" htmlFor="to">Destino</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">store</i>
                        <select  value={firstStop} onChange={ handleLineInputChange } ref={stopOriginMultipleSelect} name="firstStop" >
                            <option value="Elige una opción para parada origen"  defaultValue='Elige una opción para parada origen'>Elige una opción para parada origen</option>
                            {
                                allStops.map(stop => (<option key={stop._id} value={stop._id}>{stop.codeStop} - {stop.name} </option> ))
                            }                            
                        </select>
                        <label className="active" htmlFor="from">Parada de origen</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">store</i>
                        <select  value={lastStop} onChange={ handleLineInputChange } ref={stopDestinationMultipleSelect} name="lastStop" >
                            <option value="Elige una opción para parada origen"  defaultValue='Elige una opción para parada origen'>Elige una opción para parada destino</option>
                            {
                                allStops.map(stop => (<option key={stop._id} value={stop._id}>{stop.codeStop} - {stop.name} </option> ))
                            }                            
                        </select>
                        
                        <label className="active" htmlFor="to">Parada de destino</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12 ">
                        <i className="material-icons prefix">color_lens</i>
                        <input id="colorStart" autoComplete="colorStart" type="color" className="validate" name="colorStart" value={colorStart} onChange={ handleLineInputChange } />

                        <label className="active" htmlFor="colorStart">Color degradado inicio</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12 ">
                        <i className="material-icons prefix">color_lens</i>
                        <input id="colorEnd" autoComplete="colorEnd" type="color" className="validate" name="colorEnd" value={colorEnd} onChange={ handleLineInputChange } />
                        <label className="active" htmlFor="colorEnd">Color degradado fin</label>
                    </div>
                </div>                
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">store</i>
                        <select ref={selectStops} multiple name="stops" onChange={handleStopsMultipleSelect}>
                            <option value="" disabled defaultValue="Elige una opción">Elige una opción</option>
                                {
                                    allStops.map(stop => {
                                        if(modal.data?.stops.includes(stop?._id)){
                                            return <option key={stop?._id} value={stop?._id} defaultValue={`${stop?.codeStop}`} selected  >{stop?.codeStop} - {stop?.name} </option>
                                        }
                                        else{
                                            return <option key={stop?._id} value={stop?._id}>{stop?.codeStop} - {stop?.name} </option>
                                        }
                                        
                                    })
                                }                            
                        </select>
                        <label>Selecciona todas las paradas que tiene esta línea</label>
                    </div>
                </div>
                <button className="btn waves-effect waves-light teal green" type="button" onClick={addActivity}>
                    <i className="material-icons white-text text-darken-4 left ">add</i> Agregar Fecha Servicio
                </button>
                {fechasServicio}
                            
                
                <br />
                <div className="center-align animate__fadeInDown">
                    <button className="btn waves-effect waves-light teal modal-close" type="submit" name="action">
                        <i className="material-icons white-text text-darken-4 left ">save</i>Guardar
                    </button>
                    <button to="#!" className="ml15 modal-close red darken-red-3 waves-effect waves-red btn" onClick={resetForm}>
                        <i className="material-icons left">cancel</i>Cancelar
                    </button>
                </div>
            </div>
            </form>
        </div>
    )
}
