import React, { useEffect, useRef } from 'react'
import { useForm } from '../../../hooks/useForm';
import M from "materialize-css/dist/js/materialize.min.js";
import {useDispatch, useSelector } from 'react-redux';
import { closeModal } from '../../../actions/modal';
import { startCloseFormMode } from '../../../actions/form';
import { notificationSavingNew } from '../../../actions/notification';
import { Link } from 'react-router-dom';
import Swal from 'sweetalert2';

export const CrudLinesNotificationsForm = ({allLines}) => {
    const form = useSelector(state => state.form)

    // Inicialización de formulario
    const [ formLineValues, handleLineInputChange, reset ] = useForm({
        description : '',
        line : '',
    });

    const {
        description,
        line,
    } = formLineValues;


    const dispatch = useDispatch()    
    const selectLines = useRef();
    const pathRef = useRef();    

    // Cargamos materializecss
    useEffect(() => {
        //const elems = document.querySelectorAll('select');        
        M.FormSelect.init(selectLines.current, {});
        return () => {
            //selectStops.current.destroy();
        }
    })
    
    // Envío de datos al server
    const handleLineSubmit = (e) => {
        e.preventDefault();        
        // Recorre todos los periodos de actividad y los transforma en arrays normales
        // Se preparan los stops seleccionados
        const auxResultado = allLines.find(liney => liney._id === line);
        if(auxResultado){        
            const {codeLine} = auxResultado;
            // Preparamos lo que vamos a enviar
            const datosAMandar = {description, notificationType: 'Line', busServiceId: line, codeLine }

            // Despachamos en función de lo que hagamos (Editar | Nuevo)        
            if(form.new) {
                dispatch(notificationSavingNew(datosAMandar))
            }        
            // RESET antes de irse
            resetForm()
        }
        else{
            Swal.fire('Faltan datos', 'Rellena todos los campos', 'warning');
        }
    }

    const resetForm = (e)=>{
        e?.preventDefault();
        reset() // para el        
        // Reseteo el selector de rutas
        selectLines.current.selectedIndex = -1;        
        dispatch(closeModal());
        dispatch(startCloseFormMode())        
    }
    
    const changedValue = () => {
        pathRef.current.style.overflowY = "scroll";
    }
        
    
    return (
        <div id="LinesForm" className="col s12">
            <form className="col s12" onSubmit={ handleLineSubmit }>
            
            <div className="form-container">
                <h3 className="teal-text">Notificación de línea</h3>
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">directions_bus</i>
                        <select ref={selectLines} name="line" onChange={handleLineInputChange} value={line}>
                            <option value="" disabled defaultValue="Elige una opción">Elige una opción</option>
                                {
                                    allLines.map(line => (
                                         <option key={line?._id} value={line?._id}>{line?.codeLine} - {line?.from} - {line?.to} </option>
                                    ))
                                }                            
                        </select>
                        <label>Selecciona todas las líneas que tiene esta ruta</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12 ">
                        <i className="material-icons prefix">description</i>
                        <textarea ref={pathRef} id="description" autoComplete="description" rows="10" className="materialize-textarea" name="description" value={description} onChange={ handleLineInputChange } onClick={changedValue} />
                        <label className="active" htmlFor="description">Descripción.</label>
                    </div>
                </div>    
                <br />
                <div className="center-align animate__fadeInDown">
                    <button className="btn waves-effect waves-light teal modal-close" type="submit" name="action">
                        <i className="material-icons white-text text-darken-4 left ">save</i>Guardar
                    </button>
                    <Link to="#!" className="ml15 modal-close red darken-red-3 waves-effect waves-red btn" onClick={resetForm}>
                        <i className="material-icons left">cancel</i>Cancelar
                    </Link>
                </div>
            </div>
            </form>
        </div>
    )
}
