import React, { useEffect, useRef } from 'react'
import { useForm } from '../../../hooks/useForm';
import M from "materialize-css/dist/js/materialize.min.js";
import {useDispatch, useSelector } from 'react-redux';
import { closeModal } from '../../../actions/modal';
import { startCloseFormMode, startDataReadyFormMode } from '../../../actions/form';
import { routeSavingNew, startEditingRoute } from '../../../actions/routes';

export const CrudRoutesForm = ({allLines}) => {
    const modal = useSelector(state => state.modal)
    const form = useSelector(state => state.form)

    // Inicialización de formulario
    const [ formLineValues, handleLineInputChange, reset, setValuesManually ] = useForm({
        description : '',
        lines : '',
        path : ''    
    });

    const {
        description,
        path
    } = formLineValues;


    const dispatch = useDispatch()
    const linesMultipleSelect = useRef([])    
    const selectLines = useRef();
    const pathRef = useRef();

    const handlelinesMultipleSelect = (e) => {                
        let arr = [];
        for(let option in e.target.selectedOptions){            
            e.target.selectedOptions[option]?.value !== undefined && (arr = [...arr, e.target.selectedOptions[option]?.value])
        }
        linesMultipleSelect.current = arr;
    }

    // Cargamos materializecss
    useEffect(() => {
        M.FormSelect.init(selectLines.current, {});
    })
    
    // Envío de datos al server
    const handleLineSubmit = (e) => {
        e.preventDefault();        
        // Recorre todos los periodos de actividad y los transforma en arrays normales
        // Se preparan los stops seleccionados
        const linesSelected = linesMultipleSelect.current

        // Preparamos lo que vamos a enviar
        const datosAMandar = {...formLineValues, lines: linesSelected}

        // Despachamos en función de lo que hagamos (Editar | Nuevo)
        if(form.edit){
            dispatch(startEditingRoute(datosAMandar))
        }
        else if(form.new) {
            dispatch(routeSavingNew(datosAMandar))
        }        
        // RESET antes de irse
        resetForm()
    }

    const resetForm = (e)=>{
        e?.preventDefault();
        reset() // para el
        //serviceTimeRef.current = [];
        linesMultipleSelect.current = []

        // Reseteo el selector de rutas
        selectLines.current.selectedIndex = -1;

        //setfechasServicio([]);
        dispatch(closeModal());
        dispatch(startCloseFormMode())
        
    }
    
    const changedValue = () => {
        pathRef.current.style.overflowY = "scroll";
    }
        
    /* Modo Edición y cargamos valores */
    useEffect(() => {
        // Aquí cargo datos si el formulario aún no está ready 
        // (si estamos en modo edición y además hay actividades)
        if( modal.data && form.edit && !form.dataReady){
            // Convierto a objeto el array de period of time para su tratamiento
            // Ya que es mejor utilizar campos únicos que claves numéricas procedentes
            // del index de cada posición del array

            // Cargo el formulario con los actividades del modal
            linesMultipleSelect.current = modal.data.lines;            
            setValuesManually({...modal.data, path: modal.data.path.map(coord => (coord[0]+ "," + coord[1])).join('\n') });
        }   
    }, [modal.data, setValuesManually, form, dispatch])

    /* Si está en modo edición, cargamos los selects*/
    useEffect(() => {
        if(form.edit && !form.dataReady){
            // Dejamos a ready el formulario una vez que ya está cargado
            dispatch(startDataReadyFormMode())
        }
    }, [dispatch, form, modal])

    
    return (
        <div id="LinesForm" className="col s12">
            <form className="col s12" onSubmit={ handleLineSubmit }>
            
            <div className="form-container">
                <h3 className="teal-text">Rutas</h3>
                <div className="row">
                    <div className="input-field col s12 ">
                        <i className="material-icons prefix">description</i>
                        <input id="description" autoComplete="description" type="text" className="validate" name="description" value={description} onChange={ handleLineInputChange } />

                        <label className="active" htmlFor="description">Descripción</label>
                    </div>
                </div>
                            
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">directions_bus</i>
                        <select ref={selectLines} multiple name="stops" onChange={handlelinesMultipleSelect}>
                            <option value="" disabled defaultValue="Elige una opción">Elige una opción</option>
                                {
                                    allLines.map(line => {

                                        if(modal.data?.lines.filter(l => l._id === line?._id).length){
                                            return <option key={line?._id} value={line?._id} defaultValue={`${line?._id}`} selected  >{line?.codeLine} - {line?.from} - {line?.to}  </option>
                                        }
                                        else{
                                            return <option key={line?._id} value={line?._id}>{line?.codeLine} - {line?.from} - {line?.to} </option>
                                        }
                                        
                                    })
                                }                            
                        </select>
                        <label>Selecciona todas las líneas que tiene esta ruta</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12 ">
                        <i className="material-icons prefix">timeline</i>
                        <textarea ref={pathRef} id="path" autoComplete="path" rows="10" className="materialize-textarea" name="path" value={path} onChange={ handleLineInputChange } onClick={changedValue} />
                        <label className="active" htmlFor="path">Ruta. Cada coordenada se sitúa en una línea diferente. La longitud y la latitud se separan por comas.</label>
                    </div>
                </div>    
                <br />
                <div className="center-align animate__fadeInDown">
                    <button className="btn waves-effect waves-light teal modal-close" type="submit" name="action">
                        <i className="material-icons white-text text-darken-4 left ">save</i>Guardar
                    </button>
                    <button to="#!" className="ml15 modal-close red darken-red-3 waves-effect waves-red btn" onClick={resetForm}>
                        <i className="material-icons left">cancel</i>Cancelar
                    </button>
                </div>
            </div>
            </form>
        </div>
    )
}
