import React, { useCallback, useEffect, useRef, useState } from 'react'
import { useDispatch, useSelector } from 'react-redux';
import { v4 } from 'uuid';
import { startCloseFormMode, startDataReadyFormMode } from '../../../actions/form';
import { closeModal } from '../../../actions/modal';
import { startEditingNewStop } from '../../../actions/stops';
import { stopSavingNew } from '../../../actions/stops';
import { useForm } from '../../../hooks/useForm';
import { StopsLinesSelectForm } from './StopsLinesSelectForm';


export const CrudStopsForm = ({stop}) => {
    const dispatch = useDispatch()    
    const form = useSelector(state => state.form)
    const modal = useSelector(state => state.modal)

    // Referencia de cada detalle de parada del DOM
    const stopSelRef = useRef([]);
    
    // DOM detalles
    const [domDetails, setDomDetails] = useState([]);

    // Guardamos nuevo set de detalles
    const [details, setDetails] = useState({})
   
    const [ formStopValues, handleStopInputChange, reset, setValuesManually ] = useForm({
        codeStop: 0,
        name: '',
        latitude: 0,
        longitude: 0,
        lines: [],
        connections: [],
        details: []
    });

    let {codeStop, name, latitude, longitude/*, details*/} = formStopValues;
    

    // Envío de datos al server
    const handleStopSubmit = (e) => {
        e.preventDefault();        
    
        // Recorre todos los periodos de actividad y los transforma en arrays normales
        let arrayDetails = [];
        for(let p in details.details){
            arrayDetails = [...arrayDetails, details.details[p]]
        }

        // Se crean los array lines y connections
        const uniqueLines = new Set();
        const lines = arrayDetails.filter(detail => {
            const duplicate = uniqueLines.has(detail.line);
            uniqueLines.add(detail.line);
            return !duplicate;
        }).map(detail => detail.line);

        const uniqueConn = new Set();
        const connections = arrayDetails.filter(detail => {
            const duplicate = uniqueConn.has(detail.nextStop);
            uniqueConn.add(detail.nextStop);
            return !duplicate;
        }).map(detail => detail.nextStop);


        // Preparamos lo que vamos a enviar
        const stopsAMandar = {...formStopValues, lines, connections, details: arrayDetails}

        // Despachamos en función de lo que hagamos (Editar | Nuevo)
        if(form.edit){
            dispatch(startEditingNewStop(stopsAMandar))
        }
        else if(form.new) {
            dispatch(stopSavingNew(stopsAMandar))
        }
        // RESET antes de irse
        resetForm()       
    }

    const resetForm = (e)=>{
        e?.preventDefault();
        // RESET antes de irse
        reset()
        stopSelRef.current = [];
        setDomDetails([])
        dispatch(closeModal());
        dispatch(startCloseFormMode())
        setDetails({})        
    }

    /* Se borra un detalle */
    const delDetail = useCallback((id) => {
        // Animación de borrado
        stopSelRef.current[id].className="card animate__animated animate__fadeOutDown";
        setTimeout(()=>{
            // Borramos la instancia del DOM
            setDomDetails(prev => prev.filter(detalle=> detalle.key !== id?.toString()))
            // Borramos la referencia de dicha instancia
            delete stopSelRef.current[id]

            // Se borran todos los detalles
            setDetails(prev => {
                let detallesAux = {}                
                for(let detalle in prev.details){
                    (detalle.toString() !== id.toString()) && 
                    (
                        detallesAux = {
                            ...detallesAux,
                            [detalle]: prev?.details[detalle]
                        }
                    )
                }
                return {   
                    ...prev,
                    details: detallesAux
                }
            })
        }, 1000)       
    },[])

    // Tomo el valor de los select
    const getDetailsValue = useCallback((sel) => {        
        const {id, nombre, valueSelected} = sel;
        setDetails(prev =>{
            return {   
                ...prev,
                details:{
                    ...prev.details,
                    [id]:{
                        ...((prev?.details?.hasOwnProperty(id)) && prev?.details[id]),
                        [nombre]: valueSelected 
                    }
                }
            }
        })
    },[])


    /* Añadir Detalle de parada */    
    const addDetailToDOM = useCallback( (detail, id = null) => {
        const idDetail = id ?? v4();
        setDomDetails( prev => [
            ...prev, 
            <StopsLinesSelectForm 
                ref={el => stopSelRef.current[idDetail] = el} 
                getDetailsValue={getDetailsValue}
                delDetail={delDetail} 
                key={idDetail}
                idDetail={idDetail}
                clase="card animate__animated animate__fadeInDown"
                line={detail?.line} 
                nextStop={detail?.nextStop}
            />
        ])

    }, [delDetail, getDetailsValue])

    /* Modo Edición y cargamos valores */
    useEffect(() => {
        // Aquí cargo datos si el formulario aún no está ready 
        // (si estamos en modo edición y además hay actividades)
        if( modal.data && form.edit && !form.dataReady){
            // Convierto a objeto el array de period of time para su tratamiento
            // Ya que es mejor utilizar campos únicos que claves numéricas procedentes
            // del index de cada posición del array
            const obj = Object.assign({}, modal.data.details);
            setDetails({details: obj})
            // Cargo el formulario con los actividades del modal
            setValuesManually({...modal.data });            
        }   
    }, [modal.data, setValuesManually, form, dispatch])

    /* Si está en modo edición, cargamos los selects*/
    useEffect(() => {
        if(form.edit && !form.dataReady){
            // Vamos añadiendo actividades al DOM.
            // Las actividades tienen tratamiento especial.
            for(let p in modal.data.details){
                addDetailToDOM(modal.data.details[p], p)
            }
            // Dejamos a ready el formulario una vez que ya está cargado
            dispatch(startDataReadyFormMode())
        }
    }, [dispatch, form, addDetailToDOM, modal])

    return (
        <div id="stopsForm" className="col s12">
            <form className="col s12" onSubmit={ handleStopSubmit }>
            <div className="form-container">
                <h3 className="teal-text">Parada</h3>
                <div className="row"> 

                    <div className="input-field col s12">
                        <i className="material-icons prefix">store</i>
                        <input id="codeStop" placeholder="Código de parada" autoComplete="codeStop" type="number" className="validate" name="codeStop" value={codeStop} onChange={ handleStopInputChange } />
                        <label className="active" htmlFor="codeStop">Código de parada</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">store</i>
                        <input id="name" autoComplete="name" type="text" className="validate" name="name" value={name} onChange={ handleStopInputChange } />
                        <label className="active" htmlFor="name">Nombre</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">place</i>
                        <input id="latitude" autoComplete="latitude" type="number" className="validate" name="latitude" value={latitude} onChange={ handleStopInputChange } />
                        <label className="active" htmlFor="latitude">Latitud</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">place</i>
                        <input id="longitude" autoComplete="longitude" type="number" className="validate" name="longitude" value={longitude} onChange={ handleStopInputChange } />
                        <label className="active" htmlFor="longitude">Longitud</label>
                    </div>
                </div>
                <button className="btn waves-effect waves-light teal green" type="button" onClick={()=>addDetailToDOM()}><i className="material-icons white-text text-darken-4 left">add</i> Agregar Detalle</button>
                {domDetails}
                
                <br />
                <div className="center-align animate__fadeInDown">
                    <button className="btn waves-effect waves-light teal modal-close" type="submit" name="action"><i className="material-icons white-text text-darken-4 left">save</i>Guardar</button>
                    <button to="#!" className="ml15 modal-close red darken-red-3 waves-effect waves-red btn" onClick={resetForm}>
                        <i className="material-icons left">cancel</i>Cancelar
                    </button>
                </div>
            </div>
            </form>
        </div>
    )
}
