import React, { useEffect, useRef/*, useState*/ } from 'react'
import { useForm } from '../../../hooks/useForm';
import M from "materialize-css/dist/js/materialize.min.js";
import {useDispatch, useSelector } from 'react-redux';
import { closeModal } from '../../../actions/modal';
import { startCloseFormMode, startDataReadyFormMode } from '../../../actions/form';
import { startEditingUser, userSavingNew } from '../../../actions/users';

export const CrudUserForm = () => {
    const modal = useSelector(state => state.modal)
    const form = useSelector(state => state.form)

    // Inicialización de formulario
    const [ formUserValues, handleUserInputChange, reset, setValuesManually ] = useForm({
        name: '',
        email: '',
        rol: ''
    });

    const {
        name,
        email,
        rol        
    } = formUserValues;


    const dispatch = useDispatch()   
    const selectRol = useRef();

    // Cargamos materializecss
    useEffect(() => {
        M.FormSelect.init(selectRol.current, {});
    })
    
    // Envío de datos al server
    const handleUserSubmit = (e) => {
        e.preventDefault();        

        // Preparamos lo que vamos a enviar
        //const {codeLine} = allLines.find(l => l._id === line)
        const datosAMandar = {...formUserValues, rol};//, codeLine}

        // Despachamos en función de lo que hagamos (Editar | Nuevo)
        if(form.edit){
            dispatch(startEditingUser(datosAMandar))
        }
        else if(form.new) {
            dispatch(userSavingNew(datosAMandar))
        }        
        // RESET antes de irse
        resetForm()
    }

    const resetForm = (e)=>{
        e?.preventDefault();
        reset()
        // Reseteo el selector de rutas
        selectRol.current.selectedIndex = -1;        
        dispatch(closeModal());
        dispatch(startCloseFormMode())
    }    
        
    /* Modo Edición y cargamos valores */
    useEffect(() => {
        // Aquí cargo datos si el formulario aún no está ready 
        // (si estamos en modo edición y además hay actividades)
        if( modal.data && form.edit && !form.dataReady){
            // Convierto a objeto el array de period of time para su tratamiento
            // Ya que es mejor utilizar campos únicos que claves numéricas procedentes
            // del index de cada posición del array
            setValuesManually({...modal.data})
        }   
    }, [modal.data, form, dispatch, setValuesManually])

    /* Si está en modo edición, cargamos los selects*/
    useEffect(() => {
        if(form.edit && !form.dataReady){
            // Dejamos a ready el formulario una vez que ya está cargado
            dispatch(startDataReadyFormMode())
        }
    }, [dispatch, form, modal])
    
    return (
        <div className="col s12">
            <form className="col s12" onSubmit={ handleUserSubmit }>
            
            <div className="form-container">
                <h3 className="teal-text">Usuario</h3>
                <div className="row">
                    <div className="input-field col s12 ">
                        <i className="material-icons prefix">user</i>
                        <input id="name" autoComplete="name" type="text" className="validate" name="name" value={name} onChange={ handleUserInputChange } />

                        <label className="active" htmlFor="name">Identificación</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12 ">
                        <i className="material-icons prefix">email</i>
                        <input id="email" autoComplete="email" type="text" className="validate" name="email" value={email} onChange={ handleUserInputChange } />

                        <label className="active" htmlFor="email">Identificación</label>
                    </div>
                </div>
                <div className="row">
                    <div className="input-field col s12">
                        <i className="material-icons prefix">directions_bus</i>
                        <select ref={selectRol} value={rol} onChange={ handleUserInputChange } name="rol" >
                            <option value="" disabled>Elige una opción</option>
                            {
                                (()=>{
                                    const role = ['ADMIN_ROLE', 'USER_ROLE', 'BUS_ROLE'];
                                    let container = [];
                                    role.forEach((_rol)=>{
                                        container = [ 
                                            ...container,
                                            (
                                                _rol === rol 
                                                ?
                                                <option key={_rol} value={_rol} selected>{_rol} </option>
                                                :
                                                _rol !== rol && <option key={_rol} value={_rol}>{_rol} </option>
                                            )                                            
                                        ]
                                    })
                                    return container
                                })()
                                
                            }                            
                        </select>
                        <label className="active" htmlFor="from">Rol al que pertenece</label>
                    </div>
                </div>              
                <br />
                <div className="center-align animate__fadeInDown">
                    <button className="btn waves-effect waves-light teal modal-close" type="submit" name="action">
                        <i className="material-icons white-text text-darken-4 left ">save</i>Guardar
                    </button>
                    <button to="#!" className="ml15 modal-close red darken-red-3 waves-effect waves-red btn" onClick={resetForm}>
                        <i className="material-icons left">cancel</i>Cancelar
                    </button>
                </div>
            </div>
            </form>
        </div>
    )
}
