import React, { useEffect, useRef } from 'react'
import M from "materialize-css/dist/js/materialize.min.js";
import { useDispatch, useSelector } from 'react-redux';
import { allLinesLoading } from '../../../actions/lines';
import { stopsStartLoading } from '../../../actions/stops';
import 'animate.css';

export const StopsLinesSelectForm = React.forwardRef(({getDetailsValue, idDetail, delDetail, clase, line=null, nextStop=null}, ref) => {
    const {data: lines, isDownloaded: linesIsDownloaded } = useSelector(state => state.lines);
    const {data: stops, isDownloaded: stopsIsDownloaded } = useSelector(state => state.stops);
    const dispatch = useDispatch()
    useEffect(() => {
        M.FormSelect.init(lineSelection.current);
        M.FormSelect.init(nextStopSelection.current);
    })
    useEffect(() => {
        !linesIsDownloaded && dispatch(allLinesLoading());
    }, [dispatch, linesIsDownloaded])

    useEffect(() => {
        !stopsIsDownloaded && dispatch(stopsStartLoading());
    }, [dispatch, stopsIsDownloaded])

    const handleLineValue = () =>{
        getDetailsValue({
            valueSelected: lineSelection.current.value,
            nombre: lineSelection.current.name,
            id: lineSelection.current.dataset.id
        })
    }

    const handleStopValue = () =>{
        getDetailsValue({
            valueSelected: nextStopSelection.current.value,
            nombre: nextStopSelection.current.name,
            id: nextStopSelection.current.dataset.id
        })
    }

    const lineSelection = useRef();
    const nextStopSelection = useRef();

    return (
        <div className={clase} ref={ref}>
            <div className="input-field col s12 ">                
                    
                    <select ref={lineSelection} data-id={idDetail} name="line" value={typeof(line) === 'string' ? line : line?._id || ''} onChange={handleLineValue}>
                        <option value="" disabled defaultValue=""  >Elige una opción</option>
                        {
                            linesIsDownloaded && lines.map(linex => {
                                const lineaID = typeof(line) === 'string' ? line : line?._id || '';
                                if(linex._id === lineaID){
                                    return <option key={linex?._id} value={linex?._id} defaultValue={`${linex?._id}`}  >{linex?.codeLine} </option>
                                }
                                return (<option key={linex._id} value={linex._id}>{linex.codeLine}</option>)
                            })
                        }
                    </select>
                
                <label>Línea</label>
            </div>
            <div className="input-field col s12 ">
                <select ref={nextStopSelection} data-id={idDetail} name="nextStop" value={nextStop || ''} onChange={handleStopValue}>
                    {!nextStop && <option key='stop0' disabled defaultValue>Siguiente parada</option>}
                    <option key='stop00' value={'0'} >Siguiente parada</option>
                    {
                        stopsIsDownloaded && stops.map(stop => {
                          return (  <option key={stop._id} value={stop.codeStop}>{stop.name} ({stop.codeStop})</option>)
                        })
                    }
                </select>
                <label>¿Con qué parada conecta en esta línea?</label>
            </div>
            <button className="btn waves-effect waves-light teal black" type="button" onClick={()=>{delDetail(idDetail)}}>
                <i className="material-icons red-text text-darken-4 left">delete</i>Eliminar detalle
            </button>
        </div>
        
    )
})