import { useCallback, useRef, useState } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { notificationsLoading, notificationsReset } from '../actions/notification';


export const useLineNotifications = (initialValues) => {
    
    const [notificaciones, setNotificaciones] = useState({
        linea: [],
        bus: []
    });

    const dispatch = useDispatch(); 
    const notifications = useSelector(state => state.notifications);
    
    const rendered = useRef(true)
    const filtraNotificaciones = useCallback(() => {
        // Problema para las otras líneas. Si previamente ya se han descargado las notificaciones
        // Necesito recuperar las nuevas de esa línea en concreto. Por lo que tengo que resetear
        // Las notificaciones la primera vez que entro
        if(rendered.current){
            rendered.current = false;
            dispatch(notificationsReset())
        }else if(! notifications?.isDownloaded && initialValues?.lines.length){
            dispatch(notificationsLoading(initialValues.lines));
        }
        else{            
            let notificacionesLineas = [];
            initialValues
                ?.lines
                .forEach( line => notificacionesLineas = [...notificacionesLineas, ...notifications.data.filter(notification => notification?.notificationType === 'Line' && notification?.codeLine === line)] ) 
            
            let notificacionesBuses = [];
            initialValues
                ?.lines
                .forEach( line => notificacionesBuses = [...notificacionesBuses, ...notifications.data.filter(notification => notification?.notificationType === 'Bus' && notification?.codeLine === line)])

            if(notificacionesLineas.length || notificacionesBuses.length){
                setNotificaciones({
                    linea: notificacionesLineas,
                    bus: notificacionesBuses
                })
            }
        }
    }, [notifications, initialValues, dispatch])

    return (
        {
            filtraNotificaciones,
            notificaciones
        }
    )
}
