import { useRef } from 'react';
import { useSelector, useDispatch } from 'react-redux';
import { getCurrentPosition } from '../helpers/currentPosition';
import { walkingPathLoadedNow } from '../actions/walkingPath';
import { fetchConToken } from '../helpers/fetch';

export const useWalkPath = ( initialState = {} ) => {

    const search = useSelector(state => state.search);
    const dispatch = useDispatch();
    
    const steps = useRef([])
    const duration = useRef([])

    const getStartCoords = () => {
        return new Promise ((resolve, reject) =>{            
            if(search.length > 0) {
                getCurrentPosition()
                .then(response => [response.coords.longitude, response.coords.latitude])
                .then(data =>{                                 
                    resolve(data)
                })
            }
        })
    }

    const getPath = (longLatOrigen, longLatDestino) => {
        const [longOrigen, latOrigen] = longLatOrigen;
        const [longDestino, latDestino] = longLatDestino;
        return new Promise ((resolve, reject) =>{
            console.log(`useWalkPath: walkingdirections/${longOrigen}/${latOrigen}/${longDestino}/${latDestino}`);
            fetchConToken(`walkingdirections/${longOrigen}/${latOrigen}/${longDestino}/${latDestino}`)
            .then(response => {
                if(response.ok){
                    return response.json();
                }
                else {
                    !response.ok && reject("No se ha podido acceder al recurso")
                }
            })
            .then(data => {
                dispatch(walkingPathLoadedNow(data));
                if(data?.error){
                    reject(data.error)
                }
                resolve(data)
            })
            .catch((error) => {
                console.log(error)
            });
        })    
    }

    const getStepsAndDuration = (startCoords, endCoords,) => {
        return new Promise ((resolve, reject) =>{            
            getPath(startCoords, endCoords)
            .then(json => {
                const data = json.routes[0];
                steps.current = data.legs[0].steps;
                duration.current = Math.floor(data.duration / 60);                
                resolve ( {
                    steps: steps.current,
                    duration: duration.current
                })
            }).catch (err => console.log(err))
        }).catch (err => console.log(err))
        
    }

    return {
        getStartCoords,
        getPath,
        getStepsAndDuration,
        duration$: duration.current,
        steps$: steps.current
    };

}