import React, { useEffect } from 'react'
import {
  BrowserRouter as Router,
  Switch  
} from 'react-router-dom';
import { useDispatch, useSelector } from 'react-redux';
import { LoginScreen } from '../components/auth/LoginScreen';
import { startChecking } from '../actions/auth';
import { PublicRoute } from './PublicRoute';
import { PrivateRoute } from './PrivateRoute';
import { DashboardRoutes } from './DashboardRoutes';
import { AdminRoute } from './AdminRoute';
import { AdminDashboardRoutes } from './AdminDashboardRoutes';
import Preloader from '../components/ui/Preloader';

export const AppRouter = () => {
    const dispatch = useDispatch();
    const {checking, uid, rol} = useSelector(state => state.auth);

    useEffect(() => {
      dispatch(startChecking())
    }, [dispatch]);

    if(checking) return (<Preloader />);
    
    return (
        <Router>
        <div>
          <Switch>
              <PublicRoute exact path="/login" component={LoginScreen} isAuthenticated={ !!uid }  />
              <AdminRoute path="/admin" component={AdminDashboardRoutes} isAdmin={rol === 'ADMIN_ROLE'}/>
              <PrivateRoute path="/" component={DashboardRoutes} isAuthenticated={ !!uid }  />
          </Switch>
        </div>
      </Router>
    )
    
}

