const { response } = require('express');
const bcrypt = require('bcryptjs');
const User = require('../model/User');
const { createJWT } = require('../helpers/jwt');
 
const createUser = async(req, res = response ) => {

    const { email, password } = req.body;

    try {
        let user = await User.findOne({ email });

        if ( user ) {
            return res.status(400).json({
                ok: false,
                msg: 'El user ya existe'
            });
        }

        user = new User( req.body );
    
        // Encriptar contraseña
        const salt = bcrypt.genSaltSync();
        user.password = bcrypt.hashSync( password, salt );


        await user.save();

        // Generar JWT
        const token = await createJWT( user.id, user.name, user.rol );
    
        res.status(201).json({
            ok: true,
            uid: user.id,
            name: user.name,
            rol: user.rol,
            token
        })
        
    } catch (error) {
        console.log(error)
        res.status(500).json({
            ok: false,
            msg: 'Por favor hable con el administrador'
        });
    }
}


const loginUser = async(req, res = response ) => {

    const { email, password } = req.body;

    try {
        
        const user = await User.findOne({ email });

        if ( !user ) {
            return res.status(400).json({
                ok: false,
                msg: 'El usuario no coincide con ese email'
            });
        }

        // Confirmar los passwords
        const validPassword = bcrypt.compareSync( password, user.password );

        if ( !validPassword ) {
            return res.status(400).json({
                ok: false,
                msg: 'Password incorrecto'
            });
        }

        // Generar JWT
        const token = await createJWT( user.id, user.name, user.rol );

        res.status(200).json({
            ok: true,
            uid: user.id,
            name: user.name,
            rol: user.rol,            
            token
        })


    } catch (error) {
        console.log(error);
        res.status(500).json({
            ok: false,
            msg: 'Por favor hable con el administrador'
        });
    }
}


const renewToken = async (req, res = response ) => {

    const { uid, name, rol} = req;
    const user = await User.findOne({ _id: uid })


    // Generar JWT
    const token = await createJWT( uid, name, rol );
    try{
        res.status(201).json({
            ok: true,
            uid,
            name,
            rol,
            token    
        })    
    } catch (error) {
        console.log(error);
        res.status(500).json({
            ok: false,
            msg: 'Por favor hable con el administrador'
        });
    }
}

module.exports = {
    createUser,
    loginUser,
    renewToken
}