const {Bus} = require('../model/Bus');
const { Line } = require('../model/Line');

exports.readBuses = async (req, res) =>{    
    try {
        const buses = await Bus.find().populate('line');
        if(!buses){
            return res.status(404).json({
                ok: false,
                msg: 'No se han encontrado autobuses'
            });    
        }
        res.status(200).json({
            ok: true,
            data: buses
            
        })
        console.log('autobuses devueltos');    
    } catch (error) {
        res.status(404).json({error: error.message});
    }
}

exports.createBus = async (req, res) =>{    
    const bus = new Bus(req.body);
    try {
        const busExists = await Bus.findOne({identification: req.body.identification});
        if(busExists){
            return res.status(401).json({
                ok: false,
                msg: "La identificación ya existe y no se puede repetir"
            });    
        }
        await bus.save();

        // Añadimos el bus al array de buses en Línea
        await Line.updateOne(
            {_id: bus.line},
            {
                $addToSet: { buses: bus._id }        
            }
        );

        return res.status(201).json({
            ok: true,
            data: bus
        });
    } catch (error) {
        res.status(409).json({error: error.message});
        console.log(error.message);
    }
}

exports.editBus = async (req, res) =>{ 
    const busId = req.params.id;
    try {

        let busExists = await Bus.findOne({_id : busId})        
        if(!busExists){
            res.status(400).json({
                ok: false,
                msg: 'No existe ese autobús'
            });
        }
        else{
            const newBus = {
                ...req.body
            }
    
            // Configuramos la nueva línea
            const newBusUpdated = await Bus.findByIdAndUpdate( busId, newBus, { new: true } );

            // Si el autobús que había es distinto del que hay, lo sacamos de línea.
            if(busExists.line !== newBusUpdated.line){
                await Line.updateOne(
                    {_id: busExists.line},
                    {
                        $pullAll: { buses: [busExists._id]} 
                    }
                );
                await Line.updateOne(
                    {_id: newBusUpdated.line},
                    {
                        $addToSet: { buses: newBusUpdated._id }        
                    }
                );
            }

            const busReturned = await Bus.findById(newBusUpdated._id.toString()).populate('line');
            res.status(201).json({
                ok: true,
                data: busReturned
            })
        }

    } catch (error) {
        console.log(error)
        res.status(409).json({
            ok: false,
            msg: 'Hable con el administrador'
        });
    }
}

exports.deleteBus = async (req, res) =>{ 
    const busId = req.params.id;
    try {

        let busExists = await Bus.findOne({_id : busId})        
        if(!busExists){
            res.status(400).json({
                ok: false,
                msg: 'No existe esa línea'
            });
        }
        else{
            const newBus = {
                ...req.body
            }
            
            // Borramos la línea
            await Bus.findByIdAndDelete( busId );

            await Line.updateOne(
                {_id: busExists.line},
                {
                    $pullAll: { buses: [busExists._id] }        
                }
            );

            res.status(200).json({ ok: true }); 
        }

    } catch (error) {
        console.log(error)
        res.status(409).json({
            ok: false,
            msg: 'Hable con el administrador'
        });
    }
}

exports.busConnected = async( identification ) => {
    if(identification === 'demo'){
        const filter = {};
        const update = { online: true };
        return await Bus.updateMany(filter, update);    
    }
    const filter = { identification: identification };
    const bus = await Bus.findOne(filter)        
    if(!bus || bus.online){
        return false;
    }    

    bus.online = true
    await bus.save();
}


exports.busDisConnected = async( identification ) => {
    if(identification === 'demo'){
        const filter = {};
        const update = { online: false };
        return await Bus.updateMany(filter, update);    
    }
    const filter = { identification: identification };
    const update = { online: false };
    await Bus.findOneAndUpdate(filter, update);
}

exports.busResetConnections = async() => {
    const filter = {};
    const update = { online: false };
    await Bus.updateMany(filter, update);
}