const { response } = require('express');
const User = require('../model/User');
const {Line} = require('../model/Line');
 
const mngFavLines = async(req, res = response ) => {

    const { codeLine, remove } = req.body;
    const { uid } = req;

    try {
        
        const user = await User.findOne({ _id: uid });

        if ( !user ) {
            return res.status(401).json({
                ok: false,
                msg: 'El usuario no coincide con ese email'
            });
        }  
        
        const line = await Line.findOne({codeLine: codeLine.toString()});

        if ( !line ) {
            return res.status(400).json({
                ok: false,
                msg: 'La línea no existe'
            });
        }        

        let arrayFavLines = [];

        let httpStatusCode;
        if(remove){
            arrayFavLines = await user.favoriteLines.pull(line);
            httpStatusCode = 200;
        }
        else{
            arrayFavLines = await user.favoriteLines.addToSet(line);
            httpStatusCode = 201;
        }
        await user.save();

        const arrayResult = await User.findOne({ _id: uid },{select: 'favoriteLines'}).populate([{path:'favoriteLines'}]);

        res.status(httpStatusCode).json({
            ok: true,
            data: arrayResult.favoriteLines,
        })

    } catch (error) {
        console.log(error);
        res.status(500).json({
            ok: false,
            msg: 'Por favor hable con el administrador'
        });
    }
}

const addFavLines = async(req, res = response ) => {

    const { codeLine } = req.body;
    const { uid } = req;

    try {
        
        const user = await User.findOne({ _id: uid });

        if ( !user ) {
            return res.status(401).json({
                ok: false,
                msg: 'El usuario no coincide con ese email'
            });
        }  
        
        const line = await Line.findOne({codeLine: codeLine.toString()});

        if ( !line ) {
            return res.status(400).json({
                ok: false,
                msg: 'La línea no existe'
            });
        }        
        // Se añade a favoritos
        await user.favoriteLines.addToSet(line);
        await user.save();

        const arrayResult = await User.findOne({ _id: uid },{select: 'favoriteLines'}).populate([{path:'favoriteLines'}]);

        res.status(201).json({
            ok: true,
            data: arrayResult.favoriteLines,
        })

    } catch (error) {
        console.log(error);
        res.status(409).json({
            ok: false,
            msg: 'Por favor hable con el administrador'
        });
    }
}

const delFavLines = async(req, res = response ) => {
    const { id: codeLine } = req.params;
    const { uid } = req;

    try {
        
        const user = await User.findOne({ _id: uid });

        if ( !user ) {
            return res.status(401).json({
                ok: false,
                msg: 'El usuario no coincide con ese email'
            });
        }  
        
        const line = await Line.findOne({codeLine: codeLine.toString()});

        if ( !line ) {
            return res.status(400).json({
                ok: false,
                msg: 'La línea no existe'
            });
        }      
        // Se saca de favoritos
        await user.favoriteLines.pull(line);
        await user.save();

        const arrayResult = await User.findOne({ _id: uid },{select: 'favoriteLines'}).populate([{path:'favoriteLines'}]);

        res.status(200).json({
            ok: true,
            data: arrayResult.favoriteLines,
        })

    } catch (error) {
        console.log(error);
        res.status(409).json({
            ok: false,
            msg: 'Por favor hable con el administrador'
        });
    }
}

const getFavorites = async(req, res = response ) => {

    const { uid } = req;

    try {
        const favoriteLines = await User.findOne({ _id: uid },{select: 'favoriteLines'}).populate([{path:'favoriteLines'}]);

        if ( !favoriteLines ) {
            return res.status(404).json({
                ok: false,
                msg: 'No existen líneas favoritas'
            });
        }
        
        res.status(200).json({
            ok: true,
            data: favoriteLines.favoriteLines,
        })


    } catch (error) {
        console.log(error);
        res.status(500).json({
            ok: false,
            msg: 'Por favor hable con el administrador'
        });
    }
}


module.exports = {
    mngFavLines,
    addFavLines,
    delFavLines,
    getFavorites
}