const fetch = require('node-fetch');
const {stop} = require('../model/stop');
const turf = require('@turf/turf');

exports.getStopByPlace = async (req, res) =>{      
    try {
        const {place} = req.params;
        const token = process.env.MAPBOX_TOKEN;
        const query = await fetch(
            `https://api.mapbox.com/geocoding/v5/mapbox.places/${place}%20sanlúcar%20de%20barrameda.json?limit=1&access_token=${token}`,
            { method: 'GET' }
        );        
        const jsonDirections = await query.json();
        /* Antes que nada, se comprueba si la dirección existe*/
        if(jsonDirections.features.length === 0 ){
            return res.status(404).json({
                ok: false,                
                msg: 'La dirección no existe'
            });    
        } 
        else if(!jsonDirections.features[0].place_name.includes('Sanlúcar de Barrameda')){
            return res.status(400).json({
                ok: false,
                msg: 'Los lugares propuestos no están en Sanlúcar de Barrameda'
            });    
        }
        else if(!jsonDirections.features[0].text.toUpperCase().includes(place.toUpperCase())){
            return res.status(400).json({
                ok:false,
                msg: 'Escribe bien la dirección'
            });
        }

        // Extraemos el lugar buscado
        const placeFound = jsonDirections.features[0].text;
        // Se captura las coordenadas
        const placeCoords = jsonDirections.features[0].geometry.coordinates;
        // Se obtienen todas las paradas
        let stops = await stop.find();    
        //
        // Se recorren todas las paradas y por cada una de ellas
        // se calcula su Distancia.
        const paradasDistancia = stops.map(stop => {
            const from = turf.point([stop.longitude, stop.latitude]);
            const to = turf.point(placeCoords);
            const options = {units: 'kilometers'};
    
            return {
                CodParada: stop.codeStop, 
                NombreParada: stop.name,
                Distancia: turf.distance(from, to, options),
                Lugar: placeFound,
                Coordenadas: placeCoords,
            };
        })
        // Se ordenan y se obtiene la más cercana
        .sort((a, b) => (a.Distancia > b.Distancia) ? 1 : -1)
        .map(a => {
            let dist = a.Distancia;
            if(a.Distancia < 1){
                dist = ((dist*1000).toFixed(1))
                dist = dist.toString() + " metros"
            }
            else{
                dist = dist.toFixed(1).toString() + " kilometros"
            }
            
            return {
                ...a,
                Distancia: dist
            }
        })
        
        
        // Se devuelve la parada más cercana y fin del asunto.
        res.status(200).json({
            ok: true,
            data: paradasDistancia
        });
        console.log('resultados devueltos');
    } catch (error) {
        res.status(500).json({error: error.message});
    }
}