const {Notification} = require('../model/Notification');
const {Route} = require('../model/Route');

// GET
exports.readNotification = async (req, res) =>{    
    try {
        const {code} = req.params;
        const lineas = code.split(',');
        let notifications = await Notification.find({}).populate([
            {
                path: 'busServiceId',                
                populate: [
                    {
                        path: 'codeLine',

                    },
                ],
                match: { codeLine: { $in : lineas } }
            }
          ])
        if(!notifications){
           return res.status(404).json({
                ok:false,
                msg: "No existen notificaciones para esa línea."
            }); 
        }
        notifications = notifications.filter(notification => notification.busServiceId)
        res.status(200).json({
            ok: true,
            data: notifications
        })
            
             
        console.log('1 notificación devuelta');
    } catch (error) {
        res.status(500).json({error: error.message});
    }
}

exports.readAllNotifications = async (req, res) => {
    try{
        const notifications = await Notification.find({}).sort( { codeLine: 1 } ).populate('busServiceId')
        if(!notifications){
            return res.status(404).json({
                ok:false,
                msg: "No existen notificaciones. " + error.message
            }); 
        }
        res.status(200).json({
            ok:true,
            data: notifications
        });        
        console.log('Notificaciones devueltas');
    }catch (error){
        res.status(500).json({error: error.message});
    }
}

exports.createNotification = async (req, res) =>{    
    const notification = new Notification(req.body);
    try {
        await notification.save();
        res.status(201).json({
            ok: true,
            data: notification
        });
    } catch (error) {
        res.status(409).json({error: error.message});
        console.log('error de inserción');
    }
}

exports.deleteNotification = async (req, res) =>{ 
    const notificationId = req.params.id;
    try {

        let notificationExists = await Notification.findOne({_id : notificationId})        
        if(!notificationExists){
            return res.status(400).json({
                ok: false,
                msg: 'No existe esa línea'
            });
        }
        else{            
            // Borramos la línea
            await Notification.findByIdAndDelete( notificationId );
            res.status(200).json({ ok: true }); 
        }

    } catch (error) {
        res.status(409).json({
            ok: false,
            msg: 'Hable con el administrador: ' + error.message
        });
    }
}