const {Line} = require('../model/Line');
const {Route} = require('../model/Route');

exports.readRoutes = async (req, res) =>{ 
    try {
        const route = await Route.find().populate([
            {
                path: 'lines'
            }
        ]);

        if(!route){
           return  res.status(404).json({
                ok: false,
                msg: 'No existe esa ruta'
            });
        }
        
        res.status(200).json({
            ok: true,
            data: route
        });
        console.log('resultados devueltos');
    } catch (error) {
        res.status(500).json({error: error.message})
    }
}

exports.createRoute = async (req, res) =>{
    
    const { path, lines } = req.body

    // Suponiendo que ha pasado las comprobaciones de campos
    // Vienen separadas por saltos de línea. Luego, el resultado de hacer split, 
    // lo convertimos en un array de 2 dimensiones con long y lat
    
    if(!/^(([+-]?([1-9]+\.?[0-9]*|\.[0-9]+){1},[+-]?([1-9]+\.?[0-9]*|\.[0-9]+){1})\n?)*$/.test(req.body.path)){
        return res.status(409).json({
            ok: false,
            msg: 'Error al insertar las rutas'
        });
    }
    req.body.path = path.split('\n').map(coord=> {
        const [long, lat] = coord.split(',');
        return [long, lat]
    });
    console.log(req.body);

    const route = new Route( req.body );
    try {
        // Primero sacamos las líneas indicadas de todas las rutas ya que solo pueden estar en una
        await Route.updateMany(
            {},
            {
                $pullAll: { lines: lines }        
            }
        );

        // Popularizamos lines        
        await route.populate('lines').exec

        // Guardamos la nueva ruta
        const RouteGuardado = await route.save();

        // En cada documento de la colección líneas con id especificado
        // vamos a insertar en el campo route
        // la ruta especificada en RouteGuardado.
        await Line.updateMany(
            {
                _id:
                    {
                        $in:
                            
                                lines        
                            
                    }
            },
            {
                $set: { route: [RouteGuardado._id] }        
            }
        );

        const todasLasRutas = await Route.find({}).populate('lines');


        //await RouteGuardado.populate('lines').execPopulate()
        res.status(201).json({
            ok: true,
            data: todasLasRutas//RouteGuardado
        })
    } catch (error) {
        console.log(error)
        res.status(409).json({
            ok: false,
            msg: 'Hable con el administrador: ' + error.message
        });
    }
}


exports.editRoute = async (req, res) =>{ 

    // Con la edición, tenemos un problema.
    // Cuando nos cargamos una ruta, tenemos que borrarlas del Route.lines que sea
    // El modus operandi es el siguiente:
    // 1 - Recibo ruta
    // 2 - Compruebo las líneas que tiene actualmente
    // 3 - Comparo con las que estoy mandando
    // 4 - Si hay diferencias, elimina las que no se incluyan en los nuevos datos.
    // const arrayQueEnvio = ['1', '2']
    // const arrayQueTengo = ['1', '2', '3', '4']    
    // arrayQueTengo.filter(x => !arrayQueEnvio.includes(x));
    // ['3', '4'] <-- resultados a borrar
    // 7 - El resultado, si es que sale algo, se recorre y se elimina con un forEach de cada elemento y pull en Route.lines

    if(!/^(([+-]?([1-9]+\.?[0-9]*|\.[0-9]+){1},[+-]?([1-9]+\.?[0-9]*|\.[0-9]+){1})\n?)*$/.test(req.body.path)){
        return res.status(409).json({
            ok: false,
            msg: 'Error al insertar las rutas'
        });
    }
    
    const { path, lines } = req.body;
    const RouteId = req.params.id;
    try {

        let RouteExists = await Route.findOne({_id : RouteId})        
        if(!RouteExists){
            res.status(400).json({
                ok: false,
                msg: 'No existe esa ruta'
            });
        }
        else{
            // Primero adaptamos lo que nos llega de las coordenadas
            // Esto se puede sacar a middleware.
            req.body.path = path.split('\n').map(coord=> {
                const [long, lat] = coord.split(',');
                return [long, lat]
            });

            // Primero sacamos las líneas indicadas de todas las rutas ya que solo pueden estar en una
            await Route.updateMany(
                {},
                {
                    $pullAll: { lines: lines }        
                }
            );

            // Preparo la nueva ruta
            const newRoute = {
                ...req.body
            }
    
            // Configuramos la nueva ruta
            const newRouteUpdated = await Route.findByIdAndUpdate( RouteId, newRoute, { new: true } );
            
            // Popularizamos lines        
            //await newRouteUpdated.populate('lines').exec

            // Hacemos la diferencia. 
            // Pillamos los que nos están y los que sí.
            // Buscamos las paradas a eliminar
            const lineasAEliminar = RouteExists.lines.filter(x => !newRouteUpdated.lines.includes(x));
                      
            // Quitamos las que ya no están 
            await Line.updateMany(
                { _id: { $in: lineasAEliminar } },
                { $set: { route: null } }
            )

            // Introducimos las que se han metido en líneas
            await Line.updateMany(
                { _id: { $in: lines } },
                { $set: { route: RouteExists } }
            )

            const todasLasRutas = await Route.find({}).populate('lines');

            res.status(200).json({
                ok: true,
                data: todasLasRutas
            })
        }

    } catch (error) {
        console.log(error)
        res.status(409).json({
            ok: false,
            msg: 'Hable con el administrador: ' + error.message
        });
    }
}

exports.deleteRoute = async (req, res) =>{ 
    console.log("Recibo: ", req.params.id)
    const RouteId = req.params.id;
    try {

        let RouteExists = await Route.findOne({_id : RouteId})        
        if(!RouteExists){
            return res.status(400).json({
                ok: false,
                msg: 'No existe esa ruta'
            });
        }
        else{

            /*
            En todas las líneas, se sacan todos los Route que coincidan
            con el objeto que acabamos de borrar. Entrará en el array de 
            Route de cada una de las líneas.
            */

            // Hacemos la diferencia. 
            // Pillamos los que nos están y los que sí.
            // Buscamos las paradas a eliminar
                    
            // Quitamos las que ya no están 
            await Line.updateMany(
                { _id: { $in:  RouteExists.lines  } },
                { $set: { route: null } }
            )

            // Borramos la ruta
            await Route.findByIdAndDelete( RouteId );
            res.status(200).json({ ok: true });
        }

    } catch (error) {
        console.log(error)
        res.status(409).json({
            ok: false,
            msg: 'Hable con el administrador: ' + error.message
        });
    }
}