const {stop} = require('../model/stop');
const {Graph, bfs, shortestPath} = require('../helpers/graph')
const moment = require('moment');

exports.searchRoute = async (req, res) =>{    
  try {
    
      let stops = await stop.find({}).populate([
        {
            path: 'details.line'
        },
        {
            path: 'lines'
        }            
      ])

      const {place1, place2} = req.params;

      // Deberíamos sacar solo las paradas de líneas que estén disponibles
      // Por ejemplo, si la línea ese día no está disponible, no deberíamos de
      // incluir esa parada. Pero si la parada está compartida por más líneas,
      // deberíamos incorporarla solo si la parada tiene alguna línea disponible.
      // En el backend habría que omitir las líneas no disponibles en el itinerario.
      
      // Vamos a comprobar primero que la línea tenga conexión directa
      // Con el destino (para evitar transbordos innecesarios)
      let lineas1 = stops.find(stop => stop.codeStop.toString() === place1).lines.filter(line => {
        return loopPeriod(line.periodOfActivity)
      })
      .map(e => e?.codeLine)
      let lineas2 = stops.find(stop => stop.codeStop.toString() === place2).lines.filter(line => {
        return loopPeriod(line.periodOfActivity)
      })
      .map(e => e?.codeLine)

      console.log("Lineas1: ", lineas1);
      console.log("Líneas2: " ,lineas2);

      // Se buscan líneas coincidentes para que en el caso de que existieran,
      // Se utilizara una línea directa de parada a parada
      matchesFound = lineas1.filter((val) => {
        return lineas2.includes(val)
      })
            
      // Aquí habría que comprobar si la línea seleccionada está operativa
      if(matchesFound.length > 0){
        if(/^[0-9]+A|B/.test(matchesFound[0])){            
          const numLine = matchesFound[0].replace(/\D+/g, "");

          stops = stops.filter(stop => {
            return stop.details.filter( e => e?.line?.codeLine === `${numLine}A` || e?.line?.codeLine === `${numLine}B`).length ;
          })
        }
        else{
          stops = stops.filter(stop => stop.lines.filter(line => line?.codeLine === matchesFound[0]).length)
        }          
      }
      
      // Creamos el grafo
      let graph = new Graph();
  
      // Añadimos las aritas (es un grafo dirigido)
      // Habría que sacar las líneas que tienen coincidencias en un loop
      // Y nos quedamos con la que tenga menos paradas (aunque no es lo ideal)
      // Debemos sacar solo las líneas que tengan servicio en la fecha en la que
      // se lanza la consulta.     

      const today = moment().format('DD-MM-YYYY').toString();
      const todayHour = moment().format('HH:mm').toString();
      console.log("hoy es ", today)
      //console.log("Estamos a",moment().day(moment().weekday()).format("dddd").toLowerCase());
      
      for (let parada of stops){
        for (let detalle of parada.details){
          // Sacamos la línea y el periodo de actividad de esta.
          // Si está en periodo de actividad, lo agrega al grafo, si no, no.
          for(let periodo of detalle.line.periodOfActivity){
            if(isInPeriod(periodo) ){

              graph.addEdge(parada.codeStop.toString(), detalle.nextStop.toString())
            }            
          }
        }
    }
  
      // Se aplica BFS al grafo desde el origen de paradas
      bfs(graph, stops[0].codeStop.toString());
      //console.log(stops[0].code.toString())
  
      // Buscamos el camino más corto desde una parada a otra
      let path = shortestPath(graph, place1.toString(), place2.toString());
      //console.log(path.join(' -> '));

      if(!path){
       return res.status(404).json({
          ok: false,
          msg: "Lo siento, no se ha encontrado una ruta. Comprueba los horarios de autobuses."
        });
      }
      
      //Devuelve resultado
      let resultado = path.map(codePath => ([...stops.filter(({ codeStop }) => codeStop.toString() === codePath)]));

      let result = resultado.flat();
      result = result.map(item => {
          return {
            ...item.toJSON(),
            lines: item.lines.filter(line => loopPeriod(line.periodOfActivity))
          }        
      });
      res.status(200).json({
        ok: true,
        //data: resultado.flat()
        data: result
      });

  } catch (error) {
      res.status(500).json({error: error.message});
  }  
}

const isInPeriod = (periodo) => {
  const today = moment().format('DD-MM-YYYY').toString();
  const todayHour = moment().format('HH:mm').toString();

  const {initDate, endDate, initHour, endHour, daysOfWeek} = periodo;
  if(moment(today, 'DD-MM-YYYY').isBetween(moment(initDate, 'DD-MM-YYYY'), moment(endDate, 'DD-MM-YYYY'), null, '[]') &&
    moment(todayHour, 'HH:mm').isBetween(moment(initHour, 'HH:mm'), moment(endHour, 'HH:mm'), null, '[]') &&
    daysOfWeek.includes(moment().day(moment().weekday()).format("dddd").toLowerCase())){
      return true;    
  }
}

const loopPeriod = (periodOfActivity) => {
  for(let periodo of periodOfActivity){
    if(isInPeriod(periodo)){
      return true
    }
  }
  return false;
}