const {Line} = require('../model/Line');
const {stop} = require('../model/stop');

exports.readStops = async (req, res) =>{    
    try {
        const stops = await stop.find().populate([
            {
                path: 'details.line'
            }
        ]);
        //
        if(!stops){
            return res.status(404).json({
                ok:false,
                msg: "No existen paradas"
            });    
        }
        res.status(200).json(stops);        
        console.log('resultados devueltos');
        //mongoose.connection.close();
    } catch (error) {
        res.status(500).json({error: error.message});
    }
}
exports.createStop = async (req, res) =>{        
    const Stop = new stop( req.body ); 
    try {        

        //Buscar cada línea asociada en el array details
        // e incluir la parada
        //console.log(Stop.details)
        let stopExists = await stop.findOne({codeStop: Stop.codeStop})        
        if(stopExists){
            res.status(401).json({
                ok: false,
                msg: 'Ya existe esa parada'
            });
        }
        else{
            Stop.details.forEach(detail =>{
                detail._id = Date.now();
            })
            const stopGuardado = await Stop.save();

            // Guardo los stops en Líneas
            Stop.details.forEach(async(detail) => {                
                const line = await Line.findOne({_id: detail.line});
                await line.stops.addToSet(stopGuardado);
                await line.save();    
            })

            const stopPopulated = await stop.findById(Stop._id).populate([
                {
                    path: 'details.line'
                }
            ]);

            res.status(201).json({
                ok: true,
                data: stopPopulated
            })
        }

    } catch (error) {
        console.log(error)
        res.status(409).json({
            ok: false,
            msg: 'Hable con el administrador'
        });
    }
}


exports.editStop = async (req, res) =>{ 

    // Con la edición, tenemos un problema.
    // Cuando nos cargamos una parada, tenemos que borrarlas del stop.lines que sea
    // El modus operandi es el siguiente:
    // 1 - Recibo línea
    // 2 - Compruebo las paradas que tiene actualmente
    // 3 - Comparo con las que estoy mandando
    // 4 - Si hay diferencias, elimina las que no se incluyan en los nuevos datos.
    // const arrayQueEnvio = ['1', '2']
    // const arrayQueTengo = ['1', '2', '3', '4']    
    // arrayQueTengo.filter(x => !arrayQueEnvio.includes(x));
    // ['3', '4'] <-- resultados a borrar
    // 7 - El resultado, si es que sale algo, se recorre y se elimina con un forEach de cada elemento y pull en stop.lines

    const stopId = req.params.id;
    try {

        let stopExists = await stop.findOne({_id : stopId})        
        if(!stopExists){
            res.status(400).json({
                ok: false,
                msg: 'No existe esa parada'
            });
        }
        else{
            const newStop = {
                ...req.body
            }
    
            // Configuramos la nueva parada
            const newStopUpdated = await stop.findByIdAndUpdate( stopId, newStop, { new: true } );

            res.status(201).json({
                ok: true,
                data: newStopUpdated
            })
        }

    } catch (error) {
        res.status(409).json({
            ok: false,
            msg: 'Hable con el administrador: ' + error.message
        });
    }
}

exports.deleteStop = async (req, res) =>{ 
    console.log("Recibo: ", req.params.id)
    const stopId = req.params.id;
    try {

        let stopExists = await stop.findOne({_id : stopId})        
        if(!stopExists){
            res.status(400).json({
                ok: false,
                msg: 'No existe esa parada'
            });
        }
        else{

            /*
            En todas las líneas, se sacan todos los stops que coincidan
            con el objeto que acabamos de borrar. Entrará en el array de 
            stops de cada una de las líneas.
            */

            await Line.updateMany(
                {},
                { $pullAll: { stops: [stopExists._id] } },
                { multi: true }
            );

            // Borramos la parada
            await stop.findByIdAndDelete( stopId );
            res.status(200).json({ ok: true });
        }

    } catch (error) {
        res.status(409).json({
            ok: false,
            msg: 'Hable con el administrador: ' + error.message
        });
    }
}