/*
**  Autor: Michael Laszlo
**  Descripción: Implementación de algoritmo BFS y búsqueda del camino más corto.
**  Código disponible en Stackoverflow.
**  https://stackoverflow.com/questions/32527026/shortest-path-in-javascript
**  (Recuperado el 05-12-2021)
*/

function Graph() { // De cada vértice, añadimos todos sus adyacentes en el array de adyacentes.
    var adyacentes = this.adyacentes = {}; // Key = vertice, value = array de adyacentes.
  
    this.addEdge = function (u, v) {
      if (adyacentes[u] === undefined) {  //Si no está inicializado, se inicializa como array vacío
        adyacentes[u] = [];
      }
      adyacentes[u].push(v); // Se añade v al array de adyacentes de u
    };
  
    return this;
}

function bfs(grafo, origen) {
    var cola = [ { vertice: origen, count: 0 } ],
        visitado = { origen: true },
        tail = 0;
    while (tail < cola.length) {
        var u = cola[tail].vertice,
        count = cola[tail++].count;
        grafo.adyacentes[u]?.forEach(function (v) {
        if (!visitado[v]) {
            visitado[v] = true;
            cola.push({ vertice: v, count: count + 1 });
        }
        });
    }
}

function shortestPath(grafo, origen, destino) {
    if (origen == destino) {   
        print(origen);          
        return;                 
    }                         
    var cola = [ origen ],
        visitado = { origen: true },
        predecesor = {},
        tail = 0;
    while (tail < cola.length) {
        var u = cola[tail++],  // Se saca el vértice de la cola.
            adyacentes = grafo.adyacentes[u];
        for (var i = 0; i < adyacentes?.length; ++i) {
            var v = adyacentes[i];
            if (visitado[v]) {
                continue;
            }
            visitado[v] = true;
            if (v === destino) {   // Se comprueba si la ruta se ha completado
                var ruta = [ v ];   // Si es así, se vuelve atrás sobre la ruta.
                while (u !== origen) {
                    ruta.push(u);
                    u = predecesor[u];
                }
                ruta.push(u);
                ruta.reverse();
                
                return ruta;
            }
            predecesor[v] = u;
            cola.push(v);
        }
    }
    return false;
}

module.exports = {
    Graph,
    bfs,
    shortestPath
}