const express = require('express');
const app = express();
require('dotenv').config();
const { dbConnection } = require('./database/config');
const cors  = require('cors');
const path = require('path');
const buses = require('./routes/buses');
const authRoutes = require('./routes/auth');
const adminRoutes = require('./routes/admin');
const searchResults = require('./routes/searchResults');
const stops = require('./routes/stops');
const lines = require('./routes/lines');
const walkingDirections = require('./routes/walkingDirections');
const getStopByPlace = require('./routes/getStopByPlace');
const notifications = require('./routes/notifications');
const favorites = require('./routes/favorites');
const profile = require('./routes/profile');
const routes = require('./routes/routes');
const users = require('./routes/users');
const getPlaceByCoordinates = require('./routes/getPlaceByCoordinates');

// Abrimos conexión con la base de datos
dbConnection();

// Server Model: Contiene todo el servidor de express + socket.io configurado
const Server = require('./model/server');

// En build colocamos el compilado de React (npm run build)
app.use(express.static('public/build'));

//Configuración de json, url y cors
app.use(express.json({extended: true}));
app.use(express.urlencoded({extended: true}));
app.use(cors());

//Rutas
//app.use('/api/admin', adminRoutes);
app.use('/api/auth', authRoutes);
app.use('/api/buses', buses);
app.use('/api/favorites', favorites);
app.use('/api/stopinplace', getStopByPlace);
app.use('/api/lines', lines);
app.use('/api/notifications', notifications);
app.use('/api/profile', profile);
app.use('/api/search', searchResults);
app.use('/api/stops', stops);
app.use('/api/directions', walkingDirections);
app.use('/api/routes', routes);
app.use('/api/users', users);
app.use('/api/coordsinsanlucar', getPlaceByCoordinates);

app.get('/*', function(req, res) {
    res.sendFile(path.join(__dirname, '/public/build/index.html'), function(err) {
       if (err) {
          res.status(500).send(err.message)
       }
    })
 })

// Inicializar la instancia del server
const PORT = process.env.PORT || 3000;
const server = new Server(app, PORT);
// Ejecutar el server
server.execute();
