const { response } = require('express');
const jwt = require('jsonwebtoken');

const User = require('../model/User');

const validateJWT = async ( req, res = response, next ) => {
    // x-token headers
    const token = req.header('x-token');

    if ( !token ) {
        return res.status(401).json({
            ok: false,
            msg: 'There is no token'
        });
    }

    try {                
        const { uid, name, rol } = jwt.verify(
            token,
            process.env.SECRET_JWT_SALT
        );

        // leer el user que corresponde al uid
        const user = await User.findById( uid );

        if( !user ) {
            return res.status(401).json({
                msg: 'Token no válido - user no existe DB'
            })
        }

        // Verificar si el uid tiene estado true
        if ( !user.state ) {
            return res.status(401).json({
                msg: 'Token no válido - user con estado: false'
            })
        }

        //Nos quedamos con el user
        req.user = user;
        req.uid = uid;
        req.name = name;
        req.rol = user.rol;

    } catch (error) {
        return res.status(401).json({
            ok: false,
            msg: 'Not valid token'
        });
    }

    next();
}

module.exports = {
    validateJWT
}