const mongoose = require('mongoose');

const lineSchema = new mongoose.Schema({
    codeLine: {
        type: String,
        required: true,
        unique: true,
        index: true
    },
    from:{
        type:String,
        required:true,
        unique:false,
        index:false,
    },
    to:{
        type:String,
        required:true,
        unique:false,
        index:false,
    },

    colorStart:{
        type: String,
        required: false,        
        index: false,
    },
    colorEnd:{
        type: String,
        required: false,        
        index: false,
    },

    periodOfActivity:[{
        initDate: {
            type: String
        },
        endDate: {
            type: String
        },
        daysOfWeek:{
            type: [String],
            enum: ['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday']
        },
        initHour: {
            type: String,
            default: '7:30'
        },
        endHour: {
            type: String,
            default: '23:30'
        }
    }],
    firstStop:{
        type: mongoose.Schema.Types.ObjectId,
        ref: 'stop',
        required: false
    },
    lastStop:{
        type: mongoose.Schema.Types.ObjectId,
        ref: 'stop',
        required: false
    },
    stops: [{
        type: mongoose.Schema.Types.ObjectId,
        ref: 'stop',
        required: false        
    }],
    buses:[{
        type: mongoose.Schema.Types.ObjectId,
        ref: 'Bus',
        required: false
    }],
    route: {
        type: mongoose.Schema.Types.ObjectId,
        ref: 'Route',
        required: false,
        default: null
    }

});

exports.Line = mongoose.model('Line', lineSchema);