const { checkJWT } = require('../helpers/jwt');
const Marcadores = require('./marcadores');
const User = require('../model/User');
const { busConnected, busDisConnected, busResetConnections } = require('../controller/buses');


class Sockets {

    constructor ( io ) {

        this.io = io;
        this.marcadores = new Marcadores();
        const conections = async () => {
            await busResetConnections();
            this.socketEvents();
        }
        conections();
    }

    socketEvents() {
        this.io.on('connection', async ( socket ) => {
            console.log("Nuevo socket: ", socket.id);
            const [ok, uid, rol] = checkJWT(socket.handshake.query['x-token']);
            
            socket.handshake.query['bus'] && await busConnected(socket.handshake.query['bus'].toString());

            console.log(socket.handshake.query['bus']);
            if(!ok){
                console.log('No validado');
                socket.disconnect();
            }

            const user = await User.findById( uid );
            if( !user ) {
                console.log('No validado');
                socket.disconnect();
            }
           
            console.log(`Validado: ${uid} con rol ${rol}`);

            socket.on( 'mostrar-activos', () => {
                socket.emit( 'marcadores-activos' , this.marcadores.activos );
            })
            
            socket.on( 'marcador-nuevo', ( marcador ) => {
                this.marcadores.agregarMarcador( marcador );
                socket.broadcast.emit( 'marcador-nuevo', marcador )
                socket.sendBuffer = [];
            });

            if(rol === 'BUS_ROLE' && ok){                             
                socket.on( 'marcador-actualizado', (marcador) => {
                    this.marcadores.actualizarMarcador( marcador );
                    socket.emit( 'marcador-actualizado', marcador );
                    socket.broadcast.emit( 'marcador-actualizado', marcador );
                });
                socket.on('marcador-borrado', async (identification) => {
                    if(identification === 'demo'){                        
                        for(let marcador in this.marcadores.activos){
                            console.log(this.marcadores.activos[marcador].id);                            
                            socket.broadcast.emit( 'marcador-borrado', this.marcadores.activos[marcador].id );
                            this.marcadores.removerMarcador(this.marcadores.activos[marcador].id);
                        }
                        socket.emit( 'marcador-borrado', identification );
                    }
                    else {
                        socket.emit( 'marcador-borrado', identification );
                        socket.broadcast.emit( 'marcador-borrado', identification );
                        this.marcadores.removerMarcador( identification );
                    }
                });
            
                // Limpiar cuando bus desconectado
                socket.on('disconnect', async (identification) => {
                    await busDisConnected(socket.handshake.query['bus'].toString());
                })
            }
            
        
        });
    }
}
module.exports = Sockets;