const express = require('express');
const { createBus, readBuses, editBus, deleteBus } = require('../controller/buses');
const { validateJWT } = require('../middlewares/validate-jwt');
const { isAdminRole } = require('../middlewares/validate-roles');
const { check } = require('express-validator');
const { validateFields } = require('../middlewares/validate-fields');

const router = express.Router();
router.get('/', validateJWT, readBuses);

// Para admin
router.post('/',[
    validateJWT,
    isAdminRole,
    check('identification', 'Identificación es obligatorio').not().isEmpty(),
    check('line', 'Línea es obligatorio').not().isEmpty(),
    check('codeLine', 'El código de línea es obligatorio').not().isEmpty(),
    validateFields
    ], 
createBus);

router.put('/:id', [
    validateJWT,
    isAdminRole,
    check('identification', 'Identificación es obligatorio').not().isEmpty(),
    check('line', 'Línea es obligatorio').not().isEmpty(),
    check('codeLine', 'El código de línea es obligatorio').not().isEmpty(),
    validateFields
], editBus);

router.delete('/:id', [
    validateJWT, 
    isAdminRole,
    check('id', 'El id es obligatorio').not().isEmpty(),
    validateFields
], deleteBus);

module.exports = router;