const express = require('express');
const { check } = require('express-validator');
const { createLine, readAllLines, readLine, editLine, deleteLine } = require('../controller/lines');
const { validateFields } = require('../middlewares/validate-fields');
const { validateJWT } = require('../middlewares/validate-jwt');
const { isAdminRole } = require('../middlewares/validate-roles');

const router = express.Router();
router.get('/',validateJWT, readAllLines);
router.get('/:codeLine',validateJWT, readLine);

/* Estos tres van para admin */
router.post('/', [
    validateJWT,
    isAdminRole,    
    check('codeLine', 'El código de línea es obligatorio').not().isEmpty(),
    check('from', 'El origen es obligatorio').not().isEmpty(),
    check('to', 'El destino es obligatorio').not().isEmpty(),
    check('colorStart', 'Color de inicio es obligatorio').not().isEmpty(),
    check('colorEnd', 'Color de finalización es obligatorio').not().isEmpty(),
    check('periodOfActivity', 'El periodo de actividad debe estar compuesto por valores almacenados en un array').isArray(),
    check('firstStop', 'La primera parada es obligatoria').not().isEmpty(),
    check('lastStop', 'La última parada es obligatoria').not().isEmpty(),
    check('stops', 'La parada debe estar compuesta por valores almacenados en un array').isArray(),
    validateFields
], createLine); 
router.put('/:id', [
    validateJWT,
    isAdminRole,    
    check('id', 'El id es obligatorio').not().isEmpty(),
    check('codeLine', 'El código de línea es obligatorio').not().isEmpty(),
    check('from', 'El origen es obligatorio').not().isEmpty(),
    check('to', 'El destino es obligatorio').not().isEmpty(),
    check('colorStart', 'Color de inicio es obligatorio').not().isEmpty(),
    check('colorEnd', 'Color de finalización es obligatorio').not().isEmpty(),
    check('periodOfActivity', 'El periodo de actividad debe estar compuesto por valores almacenados en un array').isArray(),
    check('firstStop', 'La primera parada es obligatoria').not().isEmpty(),
    check('lastStop', 'La última parada es obligatoria').not().isEmpty(),
    check('stops', 'La parada debe estar compuesta por valores almacenados en un array').isArray(),
    validateFields
], editLine);
router.delete('/:id', [
    validateJWT,
    isAdminRole,    
    check('id', 'El id es obligatorio').not().isEmpty(),
    validateFields
], deleteLine);

module.exports = router;