const express = require('express');
const { check } = require('express-validator');
const { readAllNotifications, readNotification, createNotification, deleteNotification } = require('../controller/notifications');
const { validateFields } = require('../middlewares/validate-fields');
const { validateJWT } = require('../middlewares/validate-jwt');
const { isAdminRole, hasRole } = require('../middlewares/validate-roles');

const router = express.Router();
router.get('/', validateJWT, readAllNotifications);
router.get('/:code', validateJWT, readNotification);

// Para admin
router.post('/', [
    validateJWT,
    hasRole('ADMIN_ROLE', 'BUS_ROLE'),
    check('description', 'La descripción es obligatoria').not().isEmpty(),
    check('notificationType', 'El tipo de notificación es obligatoria').not().isEmpty(),
    check('busServiceId', '¿Se notifica a línea o a autobús?').not().isEmpty(),
    check('codeLine', 'El código de línea es obligatorio').not().isEmpty(),
    validateFields
], createNotification); 

router.delete('/:id', [
    validateJWT,
    isAdminRole,
    check('id', 'El id es obligatorio').not().isEmpty(),    
    validateFields
], deleteNotification);

module.exports = router;