const express = require('express');
const { createRoute, readRoutes, editRoute, deleteRoute } = require('../controller/routes');
const { check } = require('express-validator');
const { validateFields } = require('../middlewares/validate-fields');
const { validateJWT } = require('../middlewares/validate-jwt');
const { isAdminRole } = require('../middlewares/validate-roles');


const router = express.Router();
router.get('/', validateJWT, readRoutes);

/* Estos van para admin */
router.post('/',
[
    isAdminRole,
    validateJWT,
    check('description', 'El nombre es obligatorio').not().isEmpty(),    
    check('lines', 'Líneas debe ser de tipo array').isArray(),
    check('path', 'La ruta es obligatoria, y cada coordenada, separada por comas, con decimales delimitados por puntos y en una línea diferente.').not().isEmpty(),    
    validateFields    
],
createRoute);

router.put('/:id', [
    validateJWT,
    isAdminRole,
    check('id', 'El id es obligatorio').not().isEmpty(),    
    check('description', 'El nombre es obligatorio').not().isEmpty(),    
    check('lines', 'Líneas debe ser de tipo array').isArray(),
    check('path', 'La ruta es obligatoria, y cada coordenada, separada por comas, con decimales delimitados por puntos y en una línea diferente.').not().isEmpty(),    
    validateFields    
], editRoute);
router.delete('/:id', [
    validateJWT,
    isAdminRole,    
    check('id', 'El id es obligatorio').not().isEmpty(),     
    validateFields    
], deleteRoute);

module.exports = router;