const express = require('express');
const { createStop, readStops, editStop, deleteStop } = require('../controller/stops');
const { check } = require('express-validator');
const { validateFields } = require('../middlewares/validate-fields');
const { validateJWT } = require('../middlewares/validate-jwt');
const { isAdminRole } = require('../middlewares/validate-roles');


const router = express.Router();
router.get('/', validateJWT, readStops);

/*Esto va a admin*/
router.post('/',
[
    validateJWT,
    isAdminRole,
    check('name', 'El nombre es obligatorio').not().isEmpty(),
    check('codeStop', 'El código es obligatorio').not().isEmpty(),
    check('codeStop', 'El código debe ser numérico').isNumeric(),
    check('details', 'Detalles debe ser de tipo array de objetos').isArray(),
    check('latitude', 'Latitud debe ser un número').isNumeric(),
    check('longitude', 'Latitud debe ser un número').isNumeric(),
    validateFields
],
createStop);

router.put('/:id', [
    validateJWT,
    isAdminRole,
    check('id', 'El id es obligatorio').not().isEmpty(),
    check('name', 'El nombre es obligatorio').not().isEmpty(),
    check('codeStop', 'El código es obligatorio').not().isEmpty(),
    check('codeStop', 'El código debe ser numérico').isNumeric(),
    check('details', 'Detalles debe ser de tipo array de objetos').isArray(),
    check('latitude', 'Latitud debe ser un número').isNumeric(),
    check('longitude', 'Latitud debe ser un número').isNumeric(),
    validateFields
], editStop);

router.delete('/:id', [
    validateJWT,
    isAdminRole,
    check('id', 'El id es obligatorio').not().isEmpty(),
    validateFields    
], deleteStop);

module.exports = router;