const express = require('express');
const { check } = require('express-validator');
const { readUsers, editUser, deleteUser } = require('../controller/users');
const { validateFields } = require('../middlewares/validate-fields');
const { validateJWT } = require('../middlewares/validate-jwt');
const { isAdminRole } = require('../middlewares/validate-roles');

const router = express.Router();

// Para admin
router.get('/', [validateJWT, isAdminRole], readUsers);
router.put('/:id', [    
    validateJWT,
    isAdminRole, 
    check('id', 'El id es obligatorio').not().isEmpty(),    
    check('name', 'El nombre es obligatorio').not().isEmpty(),    
    check('email', 'El campo email debe tener un formato correcto').isEmail(),
    check('email', 'El campo email debe tener un formato correcto').not().isEmpty(),
    check('password', 'La password no puede estar vacía').not().isEmpty(),       
    validateFields    
], editUser);
router.delete('/:id', [    
    validateJWT,
    isAdminRole,
    check('id', 'El id es obligatorio').not().isEmpty(),       
    validateFields    
] , deleteUser);

module.exports = router;